@@ -76,46 +76,31 @@ static int pcl818_read(
 ) {
    int i;
    int cardnum;
-   unsigned char c;
+   unsigned int c;
 
    unsigned int minor;
 
 
    cardnum = PCL818_CARD_NUM(MINOR(inode->i_rdev));
    minor = MINOR(inode->i_rdev) - PCL818_MINOR_START(cardnum);
-   
-   
+
+   if(verify_area(VERIFY_READ,buffer,count)==-EFAULT){
+      printk(PCL818_ID " Got invalid buffer !\n");
+      return(-EFAULT);
+   }
+
    if(minor == PCL818_MINOR_DIGITAL) {
-      /*      for(i = 0; i < count; i ++) {
-	 c = PCL818_DIGITAL_GET(cardnum);
-
-	 if(!verify_area(VERIFY_WRITE,buffer,sizeof(char)))
-	   put_fs_byte(c, buffer);
-	 
-	 DPRINTK3("card %c, digital lines, read 0x%02x\n", 'A' + cardnum, c);
-	 
-	 buffer ++;
+      for(i = 0; i+1 < count; i+=2) {
+	 cli();
+	 c=PCL818_DIGITAL_GET(cardnum);
+	 put_fs_word(c,buffer);
+	 DPRINTK3("card %c, digital lines, read 0x%04x\n", 'A' + cardnum, c);
+	 sti();
+	 buffer+=2;
       }
-
-      return(count);*/
+      return(i);
    }
-
-
-   if(minor >= PCL818_MINOR_CT0) {
-/*      if(count != 2) {
-         return -ENODATA;
-      }
-
-      minor -= PCL818_MINOR_CT0;
-
-      outb((minor << 6) | PCL818_CT_LATCH_CMD, pcl818_card[pcl818_num_cards].baseaddress + PCL818_CT_CONTROL);
-      put_fs_byte(PCL818_CT_GET(minor, cardnum), buffer);
-      put_fs_byte(PCL818_CT_GET(minor, cardnum), buffer + 1);
-
-      return(count);*/
-   }
-
-
+   
    if(minor <= PCL818_MINOR_ADC15) {
       down(&pcl818_card[cardnum].sem);
 
@@ -137,18 +122,14 @@ static int pcl818_read(
          DPRINTK1("out of critical code\n");
          DPRINTK3("card %c, adc %d woken up\n", 'A' + cardnum, minor);
 
-	 PCL818_ADC_RESET_INT_REQ(cardnum);
+	 PCL818_ADC_RESET_INT_REQ(cardnum); //convert again - to reduce crosstalk
 	 PCL818_ADC_START_CONVERSION(cardnum);	 
 	 sleep_on(&pcl818_card[cardnum].wait_q);
 
-	 if(!verify_area(VERIFY_WRITE,buffer,2)){
-	    if(pcl818_card[cardnum].adcmode[minor]>3 && pcl818_card[cardnum].adcmode[minor]<8)
-	      put_fs_word(PCL818_ADC_GET(cardnum), buffer);
-	    else
-	      put_fs_word((short)(PCL818_ADC_GET(cardnum))-(short)(2047), buffer);
-	 }
-//	 put_fs_byte(PCL818_ADC_GETL(cardnum), buffer);
-//	 put_fs_byte(PCL818_ADC_GETH(cardnum), buffer+1);	 
+	 if(pcl818_card[cardnum].adcmode[minor]>3 && pcl818_card[cardnum].adcmode[minor]<8)
+	   put_fs_word(PCL818_ADC_GET(cardnum), buffer);
+	 else
+	   put_fs_word((short)(PCL818_ADC_GET(cardnum))-(short)(2047), buffer);
       }
 /*      else {
 	 DPRINTK3("card %c, adc %d, enabling ATO\n", 'A' + cardnum, minor);
@@ -193,8 +174,6 @@ static int pcl818_read(
 }
 
 
-
-
 static int pcl818_write(
    struct inode *inode,
    struct file *file,
@@ -214,6 +193,12 @@ static int pcl818_write(
    cardnum = PCL818_CARD_NUM(minor);
    minor -= PCL818_MINOR_START(cardnum);
 
+   if(verify_area(VERIFY_WRITE,buffer,count)==-EFAULT){
+      printk(PCL818_ID " Got invalid buffer !\n");
+      return(-EFAULT);
+   }
+   
+   
    if((minor == PCL818_MINOR_DAC0) || (minor == PCL818_MINOR_DAC1)) {
       minor -= PCL818_MINOR_DAC0;
       for(i = 0; i+1 < count; i+=2) {
@@ -227,6 +212,18 @@ static int pcl818_write(
       return(count);
    }
 
+   if(minor == PCL818_MINOR_DIGITAL) {
+      for(i = 0; i+1 < count; i+=2) {
+	 
+	 cli();
+	 pcl818_card[cardnum].diglines = get_fs_word(buffer);
+	 PCL818_DIGITAL_SET(pcl818_card[cardnum].diglines,cardnum);
+   	 DPRINTK3("card %c, digital lines set to 0x%04x\n", 'A' + cardnum, pcl818_card[cardnum].diglines);
+	 sti();
+	 buffer+=2;
+      }
+      return(i);
+   }
 
 /*  To make the compiler happy:                                              */
    return(0);
