@@ -21,22 +21,33 @@
 #define YY_SamplinParser_ERROR_BODY =0;
 #define YY_SamplinParser_DEBUG  1
 #define YY_SamplinParser_LEX_BODY  = 0
+#define YY_SamplinParser_CONSTRUCTOR_CODE  \
+cmp_flag=0; \
+//yydebug=1;
 #define YY_SamplinParser_MEMBERS  \
 public: \
+virtual void yyunput(int)=0; \
 virtual void create_goto(char *)=0; \
 virtual void create_negate(void)=0; \
-virtual void create_pushdblsym(char *)=0; \
-virtual void create_popdblsym(char *)=0; \
+virtual void create_pushdblsymptr(char *)=0; \
+virtual void create_pushdblsym(char *, int )=0; \
+virtual void create_pushdblsym2(char *)=0; \
+virtual void create_popdblsym(char *, int )=0; \
+virtual void create_popdblsym2(char *, int)=0; \
 virtual void create_pushdbl(double)=0; \
-virtual void create_dblbin(char)=0; \
-virtual void create_dblrelop(char)=0; \
+virtual void create_pushdblp(double *)=0; \
+virtual void create_popdbl(double *)=0; \
+virtual void create_unaryassign(int,int)=0; \
+virtual void create_binop(int)=0; \
+virtual void create_relop(int)=0; \
 virtual void create_pushstrsym(char *)=0; \
 virtual void create_popstrsym(char *)=0; \
 virtual void create_concat(void)=0; \
 virtual void create_pushstr(char *)=0; \
-virtual void create_strrelop(char)=0; \
+virtual void create_strrelop(int)=0; \
 virtual void create_print(char)=0; \
 virtual void create_myread(char)=0; \
+virtual void create_myreaddlg(char)=0; \
 virtual void create_prompt(char *)=0; \
 virtual void create_myopen(double,char *)=0; \
 virtual void create_myclose(double)=0; \
@@ -48,7 +59,6 @@ virtual void create_skipper(void)=0; \
 virtual void create_nop(void)=0; \
 virtual void create_myend(void)=0; \
 virtual void create_decide(void)=0; \
-virtual void create_boole(char)=0; \
 virtual void create_function(int, char *)=0; \
 virtual void create_doarray(char *,int)=0; \
 virtual void create_dim(char *,char)=0; \
@@ -65,19 +75,17 @@ virtual void create_devrdb(char *, char *)=0; \
 virtual void create_devwr(char *,int)=0; \
 virtual void create_devwrb(char *, char *)=0; \
 virtual void create_devctl(char *)=0; \
+virtual void create_devflush(char *)=0; \
+virtual void create_devseek(char *,int)=0; \
 virtual void create_message(char)=0; \
 virtual void create_plotfnc(int,char*,char*)=0;\
 virtual void create_graphfnc(int,int,char*,char*)=0;\
-virtual void create_addplot(int, char *,char *)=0; \
-virtual void create_updateplot(int, char *, char *)=0; \
-virtual void create_delplot()=0; \
 virtual void create_addgraph()=0; \
 virtual void create_delgraph()=0; \
 virtual void create_loadgraph(int)=0; \
 virtual void create_savegraph(int)=0; \
 virtual void create_exportgraph(int)=0; \
 virtual void create_printgraph()=0; \
-virtual void create_titlegraph()=0; \
 virtual void create_adddlg(char *)=0; \
 virtual void create_deldlg(char *)=0; \
 virtual void create_hidedlg(char *)=0; \
@@ -89,6 +97,7 @@ virtual void create_multi(int )=0; \
 virtual void create_timer(int )=0; \
 virtual void create_cursor(int )=0;\
 virtual void inccounter(void)=0; \
+virtual void pushdummy(void)=0; \
 virtual void pushlabel(void)=0; \
 virtual void poplabel(void)=0; \
 virtual void swap(void)=0; \
@@ -98,10 +107,12 @@ virtual void popgoto(void)=0;\
 virtual void pushcopy(void)=0; \
 virtual void pushcounter(void)=0; \
 virtual void pushname(char *)=0; \
-virtual void error(int, const char*)=0; \
+virtual void error(int, const char*,...)=0; \
+virtual void pushint(int)=0; \
+char const *tname(int token);\
 \
 enum functions { \
-   MYRAN2,MYINKEY,MYREADY,MYSPOLL,MYACTIVE,MYDATE,MYTIME,ZEROARGS,MYSIN,MYASIN,MYCOS,MYACOS,MYTAN,MYATAN, \
+   MYRAN2,MYINKEY,MYREADY,MYSPOLL,MYACTIVE,MYDATE,MYTIME,ZEROARGS,MYABS,MYSGN,MYSIN,MYASIN,MYCOS,MYACOS,MYTAN,MYATAN, \
    MYEXP,MYLOG,MYLEN,MYSTR,MYCHR,MYSQRT,MYFRAC,MYRAN,MYINT,MYVAL,MYASC, \
    ONEARGS,MYATAN2,MYLEFT,MYRIGHT,TWOARGS,MYMID \
 }; \
@@ -109,6 +120,9 @@ enum arraymode { \
    CALLARRAY,ASSIGNARRAY,CALLSTRINGARRAY,ASSIGNSTRINGARRAY, CALLPTRARRAY, \
    CALLPTRSTRARRAY \
 }; \
+enum stackmode { \
+   SINGLE, ARRAY, PUSHBACK=0x0100, PUSHBEFORE=0x0200, PUSHAFTER=0x0400 \
+}; \
 enum plotfnc { \
 PLOTADD,PLOTDEL,PLOTSETDATA,PLOTSETDATAX,PLOTSETDATAY, \
 PLOTSETDATAL,PLOTSETDATAXL,PLOTSETDATAYL,PLOTGETDATA,PLOTAXES,PLOTSTYLE,\
@@ -122,9 +136,10 @@ int errorlevel; \
 int myflag;\
 struct command *lastcommand; \
 int yylineno; \
+int cmp_flag;\
 
 
-#line 136 "script.bison.y"
+#line 152 "script.bison.y"
 typedef union {
   double number;        /* double number */
   int token;            /* token of command */
@@ -133,6 +148,9 @@ typedef union {
   char *strsym;         /* string symbol */
   char *arraysym;       /* array symbol */
   char *arraystrsym;
+  char sep;
+//  int unaryop;
+//  int binaryop;
 } yy_SamplinParser_stype;
 #define YY_SamplinParser_STYPE yy_SamplinParser_stype
 
@@ -188,21 +206,21 @@ typedef union {
 #ifndef YY_SamplinParser_PURE
 
 /* #line 63 "/usr/local/lib/bison.h" */
-#line 192 "script.bison.h"
+#line 210 "script.bison.h"
 
 #line 63 "/usr/local/lib/bison.h"
 /* YY_SamplinParser_PURE */
 #endif
 
 /* #line 65 "/usr/local/lib/bison.h" */
-#line 199 "script.bison.h"
+#line 217 "script.bison.h"
 
 #line 65 "/usr/local/lib/bison.h"
 /* prefix */
 #ifndef YY_SamplinParser_DEBUG
 
 /* #line 67 "/usr/local/lib/bison.h" */
-#line 206 "script.bison.h"
+#line 224 "script.bison.h"
 
 #line 67 "/usr/local/lib/bison.h"
 /* YY_SamplinParser_DEBUG */
@@ -210,7 +228,7 @@ typedef union {
 #ifndef YY_SamplinParser_LSP_NEEDED
 
 /* #line 70 "/usr/local/lib/bison.h" */
-#line 214 "script.bison.h"
+#line 232 "script.bison.h"
 
 #line 70 "/usr/local/lib/bison.h"
  /* YY_SamplinParser_LSP_NEEDED*/
@@ -288,7 +306,7 @@ extern YY_SamplinParser_STYPE YY_SamplinParser_LVAL;
 
 
 /* #line 143 "/usr/local/lib/bison.h" */
-#line 292 "script.bison.h"
+#line 310 "script.bison.h"
 #define	DECNUMBER	258
 #define	HEXNUMBER	259
 #define	SYMBOL	260
@@ -297,145 +315,180 @@ extern YY_SamplinParser_STYPE YY_SamplinParser_LVAL;
 #define	ARRAYSYM	263
 #define	ARRAYSTRSYM	264
 #define	SEP	265
-#define	FOR	266
-#define	TO	267
-#define	STEP	268
-#define	NEXT	269
-#define	GOTO	270
-#define	GOSUB	271
-#define	LABEL	272
-#define	ON	273
-#define	IF	274
-#define	THEN	275
-#define	ELSE	276
-#define	ENDIF	277
-#define	PRINT	278
-#define	INPUT	279
-#define	RETURN	280
-#define	DIM	281
-#define	END	282
-#define	AND	283
-#define	OR	284
-#define	NOT	285
-#define	NE	286
-#define	LE	287
-#define	GE	288
-#define	LT	289
-#define	GT	290
-#define	EQ	291
-#define	READ	292
-#define	DATA	293
-#define	RESTORE	294
-#define	OPEN	295
-#define	CLOSE	296
-#define	WINDOW	297
-#define	DOT	298
-#define	LINE	299
-#define	CIRCLE	300
-#define	TEXT	301
-#define	CLEAR	302
-#define	PRINTER	303
-#define	WAIT	304
-#define	BELL	305
-#define	EOFILE	306
-#define	SIN	307
-#define	ASIN	308
-#define	COS	309
-#define	ACOS	310
-#define	TAN	311
-#define	ATAN	312
-#define	EXP	313
-#define	LOG	314
-#define	SQRT	315
-#define	INT	316
-#define	FRAC	317
-#define	RAN	318
-#define	LEN	319
-#define	VAL	320
-#define	LEFT_	321
-#define	RIGHT_	322
-#define	MID	323
-#define	STR	324
-#define	CHR	325
-#define	INKEY	326
-#define	ASC	327
-#define	DEVRD	328
-#define	DEVWR	329
-#define	DEVOPEN	330
-#define	DEVCLOSE	331
-#define	DEVCTL	332
-#define	DEVSPOLL	333
-#define	MESSAGE	334
-#define	M_INFO	335
-#define	M_WARNING	336
-#define	M_ERROR	337
-#define	ADDXPLOT	338
-#define	ADDYPLOT	339
-#define	ADDXYPLOT	340
-#define	UPDATEXPLOT	341
-#define	UPDATEYPLOT	342
-#define	UPDATEXYPLOT	343
-#define	DELPLOT	344
-#define	TITLEGRAPH	345
-#define	ADDGRAPH	346
-#define	DELGRAPH	347
-#define	LOADGRAPH	348
-#define	SAVEGRAPH	349
-#define	EXPORTGRAPH	350
-#define	PRINTGRAPH	351
-#define	ADDPLOT	352
-#define	SETPLOTDATA	353
-#define	GETPLOTDATA	354
-#define	SETPLOTAXES	355
-#define	SETPLOTSTYLE	356
-#define	SETPLOTPEN	357
-#define	SETPLOTSYMBOL	358
-#define	SETGRAPHTITLE	359
-#define	SETGRAPHCOMMENT	360
-#define	SETAXISTITLE	361
-#define	SETAXISFLAGS	362
-#define	SETAXISSCALE	363
-#define	SETAXISSTYLE	364
-#define	SETGRAPHLEGEND	365
-#define	SETGRAPHCOLORS	366
-#define	AUTO	367
-#define	REF	368
-#define	INV	369
-#define	SYM	370
-#define	FLT	371
-#define	LEFT	372
-#define	RIGHT	373
-#define	TOP	374
-#define	BOTTOM	375
-#define	ADDDLG	376
-#define	DELDLG	377
-#define	HIDEDLG	378
-#define	SHOWDLG	379
-#define	PROGRESSDLG	380
-#define	ADDWIDGET	381
-#define	BUTTON	382
-#define	SWITCH	383
-#define	LCD	384
-#define	LED	385
-#define	DLGACTIVE	386
-#define	FRAME	387
-#define	WHEEL	388
-#define	SELECT	389
-#define	GRAPH	390
-#define	KNOB	391
-#define	LEVEL	392
-#define	SLIDER	393
-#define	SLEEP	394
-#define	MULTI	395
-#define	OFF	396
-#define	IDLE	397
-#define	DEVREADY	398
-#define	CURSOR	399
-#define	DEFAULT	400
-#define	TIMER	401
-#define	DATE	402
-#define	TIME	403
-#define	UMINUS	404
+#define	SPC	266
+#define	FOR	267
+#define	TO	268
+#define	STEP	269
+#define	NEXT	270
+#define	GOTO	271
+#define	GOSUB	272
+#define	LABEL	273
+#define	ON	274
+#define	IF	275
+#define	THEN	276
+#define	ELSE	277
+#define	ENDIF	278
+#define	WHILE	279
+#define	DO	280
+#define	BREAK	281
+#define	PRINT	282
+#define	INPUT	283
+#define	RETURN	284
+#define	DIM	285
+#define	END	286
+#define	PLUSPLUS	287
+#define	MINUSMINUS	288
+#define	LEFTOP	289
+#define	RIGHTOP	290
+#define	MULEQ	291
+#define	PLUSEQ	292
+#define	MINUSEQ	293
+#define	DIVEQ	294
+#define	ANDEQ	295
+#define	OREQ	296
+#define	XOREQ	297
+#define	ISEQ	298
+#define	POWEQ	299
+#define	LEFTEQ	300
+#define	RIGHTEQ	301
+#define	PLS	302
+#define	MNS	303
+#define	DVD	304
+#define	MLP	305
+#define	PWR	306
+#define	AND	307
+#define	OR	308
+#define	XOR	309
+#define	ANDAND	310
+#define	OROR	311
+#define	NOT	312
+#define	NE	313
+#define	LE	314
+#define	GE	315
+#define	LT	316
+#define	GT	317
+#define	EQ	318
+#define	READ	319
+#define	DATA	320
+#define	RESTORE	321
+#define	OPEN	322
+#define	CLOSE	323
+#define	WINDOW	324
+#define	DOT	325
+#define	LINE	326
+#define	CIRCLE	327
+#define	TEXT	328
+#define	CLEAR	329
+#define	PRINTER	330
+#define	WAIT	331
+#define	BELL	332
+#define	EOFILE	333
+#define	SIN	334
+#define	ASIN	335
+#define	COS	336
+#define	ACOS	337
+#define	TAN	338
+#define	ATAN	339
+#define	EXP	340
+#define	LOG	341
+#define	SQRT	342
+#define	ABS	343
+#define	SGN	344
+#define	INT	345
+#define	FRAC	346
+#define	RAN	347
+#define	LEN	348
+#define	VAL	349
+#define	LEFT_	350
+#define	RIGHT_	351
+#define	MID	352
+#define	STR	353
+#define	CHR	354
+#define	INKEY	355
+#define	ASC	356
+#define	DEVRD	357
+#define	DEVWR	358
+#define	DEVOPEN	359
+#define	DEVCLOSE	360
+#define	DEVCTL	361
+#define	DEVSPOLL	362
+#define	DEVSEEK	363
+#define	DEVFLUSH	364
+#define	START	365
+#define	CURRENT	366
+#define	MESSAGE	367
+#define	M_INFO	368
+#define	M_WARNING	369
+#define	M_ERROR	370
+#define	ADDXPLOT	371
+#define	ADDYPLOT	372
+#define	ADDXYPLOT	373
+#define	UPDATEXPLOT	374
+#define	UPDATEYPLOT	375
+#define	UPDATEXYPLOT	376
+#define	DELPLOT	377
+#define	TITLEGRAPH	378
+#define	ADDGRAPH	379
+#define	DELGRAPH	380
+#define	LOADGRAPH	381
+#define	SAVEGRAPH	382
+#define	EXPORTGRAPH	383
+#define	PRINTGRAPH	384
+#define	ADDPLOT	385
+#define	SETPLOTDATA	386
+#define	GETPLOTDATA	387
+#define	SETPLOTAXES	388
+#define	SETPLOTSTYLE	389
+#define	SETPLOTPEN	390
+#define	SETPLOTSYMBOL	391
+#define	SETGRAPHTITLE	392
+#define	SETGRAPHCOMMENT	393
+#define	SETAXISTITLE	394
+#define	SETAXISFLAGS	395
+#define	SETAXISSCALE	396
+#define	SETAXISSTYLE	397
+#define	SETGRAPHLEGEND	398
+#define	SETGRAPHCOLORS	399
+#define	AUTO	400
+#define	REF	401
+#define	INV	402
+#define	SYM	403
+#define	FLT	404
+#define	LEFT	405
+#define	RIGHT	406
+#define	TOP	407
+#define	BOTTOM	408
+#define	ADDDLG	409
+#define	DELDLG	410
+#define	HIDEDLG	411
+#define	SHOWDLG	412
+#define	PROGRESSDLG	413
+#define	ADDWIDGET	414
+#define	BUTTON	415
+#define	SWITCH	416
+#define	LCD	417
+#define	LED	418
+#define	DLGACTIVE	419
+#define	INPUTDLG	420
+#define	FRAME	421
+#define	WHEEL	422
+#define	SELECT	423
+#define	GRAPH	424
+#define	KNOB	425
+#define	LEVEL	426
+#define	SLIDER	427
+#define	SLEEP	428
+#define	MULTI	429
+#define	OFF	430
+#define	IDLE	431
+#define	DEVREADY	432
+#define	CURSOR	433
+#define	DEFAULT	434
+#define	TIMER	435
+#define	DATE	436
+#define	TIME	437
+#define	LOWER_THAN_ELSE	438
+#define	UMINUS	439
 
 
 #line 143 "/usr/local/lib/bison.h"
@@ -480,7 +533,7 @@ public:
 /* static const int token ... */
 
 /* #line 182 "/usr/local/lib/bison.h" */
-#line 484 "script.bison.h"
+#line 537 "script.bison.h"
 static const int DECNUMBER;
 static const int HEXNUMBER;
 static const int SYMBOL;
@@ -489,6 +542,7 @@ static const int STRING;
 static const int ARRAYSYM;
 static const int ARRAYSTRSYM;
 static const int SEP;
+static const int SPC;
 static const int FOR;
 static const int TO;
 static const int STEP;
@@ -501,13 +555,39 @@ static const int IF;
 static const int THEN;
 static const int ELSE;
 static const int ENDIF;
+static const int WHILE;
+static const int DO;
+static const int BREAK;
 static const int PRINT;
 static const int INPUT;
 static const int RETURN;
 static const int DIM;
 static const int END;
+static const int PLUSPLUS;
+static const int MINUSMINUS;
+static const int LEFTOP;
+static const int RIGHTOP;
+static const int MULEQ;
+static const int PLUSEQ;
+static const int MINUSEQ;
+static const int DIVEQ;
+static const int ANDEQ;
+static const int OREQ;
+static const int XOREQ;
+static const int ISEQ;
+static const int POWEQ;
+static const int LEFTEQ;
+static const int RIGHTEQ;
+static const int PLS;
+static const int MNS;
+static const int DVD;
+static const int MLP;
+static const int PWR;
 static const int AND;
 static const int OR;
+static const int XOR;
+static const int ANDAND;
+static const int OROR;
 static const int NOT;
 static const int NE;
 static const int LE;
@@ -539,6 +619,8 @@ static const int ATAN;
 static const int EXP;
 static const int LOG;
 static const int SQRT;
+static const int ABS;
+static const int SGN;
 static const int INT;
 static const int FRAC;
 static const int RAN;
@@ -557,6 +639,10 @@ static const int DEVOPEN;
 static const int DEVCLOSE;
 static const int DEVCTL;
 static const int DEVSPOLL;
+static const int DEVSEEK;
+static const int DEVFLUSH;
+static const int START;
+static const int CURRENT;
 static const int MESSAGE;
 static const int M_INFO;
 static const int M_WARNING;
@@ -610,6 +696,7 @@ static const int SWITCH;
 static const int LCD;
 static const int LED;
 static const int DLGACTIVE;
+static const int INPUTDLG;
 static const int FRAME;
 static const int WHEEL;
 static const int SELECT;
@@ -627,6 +714,7 @@ static const int DEFAULT;
 static const int TIMER;
 static const int DATE;
 static const int TIME;
+static const int LOWER_THAN_ELSE;
 static const int UMINUS;
 
 
@@ -636,7 +724,7 @@ static const int UMINUS;
 enum YY_SamplinParser_ENUM_TOKEN { YY_SamplinParser_NULL_TOKEN=0
 
 /* #line 185 "/usr/local/lib/bison.h" */
-#line 640 "script.bison.h"
+#line 728 "script.bison.h"
 	,DECNUMBER=258
 	,HEXNUMBER=259
 	,SYMBOL=260
@@ -645,145 +733,180 @@ enum YY_SamplinParser_ENUM_TOKEN { YY_SamplinParser_NULL_TOKEN=0
 	,ARRAYSYM=263
 	,ARRAYSTRSYM=264
 	,SEP=265
-	,FOR=266
-	,TO=267
-	,STEP=268
-	,NEXT=269
-	,GOTO=270
-	,GOSUB=271
-	,LABEL=272
-	,ON=273
-	,IF=274
-	,THEN=275
-	,ELSE=276
-	,ENDIF=277
-	,PRINT=278
-	,INPUT=279
-	,RETURN=280
-	,DIM=281
-	,END=282
-	,AND=283
-	,OR=284
-	,NOT=285
-	,NE=286
-	,LE=287
-	,GE=288
-	,LT=289
-	,GT=290
-	,EQ=291
-	,READ=292
-	,DATA=293
-	,RESTORE=294
-	,OPEN=295
-	,CLOSE=296
-	,WINDOW=297
-	,DOT=298
-	,LINE=299
-	,CIRCLE=300
-	,TEXT=301
-	,CLEAR=302
-	,PRINTER=303
-	,WAIT=304
-	,BELL=305
-	,EOFILE=306
-	,SIN=307
-	,ASIN=308
-	,COS=309
-	,ACOS=310
-	,TAN=311
-	,ATAN=312
-	,EXP=313
-	,LOG=314
-	,SQRT=315
-	,INT=316
-	,FRAC=317
-	,RAN=318
-	,LEN=319
-	,VAL=320
-	,LEFT_=321
-	,RIGHT_=322
-	,MID=323
-	,STR=324
-	,CHR=325
-	,INKEY=326
-	,ASC=327
-	,DEVRD=328
-	,DEVWR=329
-	,DEVOPEN=330
-	,DEVCLOSE=331
-	,DEVCTL=332
-	,DEVSPOLL=333
-	,MESSAGE=334
-	,M_INFO=335
-	,M_WARNING=336
-	,M_ERROR=337
-	,ADDXPLOT=338
-	,ADDYPLOT=339
-	,ADDXYPLOT=340
-	,UPDATEXPLOT=341
-	,UPDATEYPLOT=342
-	,UPDATEXYPLOT=343
-	,DELPLOT=344
-	,TITLEGRAPH=345
-	,ADDGRAPH=346
-	,DELGRAPH=347
-	,LOADGRAPH=348
-	,SAVEGRAPH=349
-	,EXPORTGRAPH=350
-	,PRINTGRAPH=351
-	,ADDPLOT=352
-	,SETPLOTDATA=353
-	,GETPLOTDATA=354
-	,SETPLOTAXES=355
-	,SETPLOTSTYLE=356
-	,SETPLOTPEN=357
-	,SETPLOTSYMBOL=358
-	,SETGRAPHTITLE=359
-	,SETGRAPHCOMMENT=360
-	,SETAXISTITLE=361
-	,SETAXISFLAGS=362
-	,SETAXISSCALE=363
-	,SETAXISSTYLE=364
-	,SETGRAPHLEGEND=365
-	,SETGRAPHCOLORS=366
-	,AUTO=367
-	,REF=368
-	,INV=369
-	,SYM=370
-	,FLT=371
-	,LEFT=372
-	,RIGHT=373
-	,TOP=374
-	,BOTTOM=375
-	,ADDDLG=376
-	,DELDLG=377
-	,HIDEDLG=378
-	,SHOWDLG=379
-	,PROGRESSDLG=380
-	,ADDWIDGET=381
-	,BUTTON=382
-	,SWITCH=383
-	,LCD=384
-	,LED=385
-	,DLGACTIVE=386
-	,FRAME=387
-	,WHEEL=388
-	,SELECT=389
-	,GRAPH=390
-	,KNOB=391
-	,LEVEL=392
-	,SLIDER=393
-	,SLEEP=394
-	,MULTI=395
-	,OFF=396
-	,IDLE=397
-	,DEVREADY=398
-	,CURSOR=399
-	,DEFAULT=400
-	,TIMER=401
-	,DATE=402
-	,TIME=403
-	,UMINUS=404
+	,SPC=266
+	,FOR=267
+	,TO=268
+	,STEP=269
+	,NEXT=270
+	,GOTO=271
+	,GOSUB=272
+	,LABEL=273
+	,ON=274
+	,IF=275
+	,THEN=276
+	,ELSE=277
+	,ENDIF=278
+	,WHILE=279
+	,DO=280
+	,BREAK=281
+	,PRINT=282
+	,INPUT=283
+	,RETURN=284
+	,DIM=285
+	,END=286
+	,PLUSPLUS=287
+	,MINUSMINUS=288
+	,LEFTOP=289
+	,RIGHTOP=290
+	,MULEQ=291
+	,PLUSEQ=292
+	,MINUSEQ=293
+	,DIVEQ=294
+	,ANDEQ=295
+	,OREQ=296
+	,XOREQ=297
+	,ISEQ=298
+	,POWEQ=299
+	,LEFTEQ=300
+	,RIGHTEQ=301
+	,PLS=302
+	,MNS=303
+	,DVD=304
+	,MLP=305
+	,PWR=306
+	,AND=307
+	,OR=308
+	,XOR=309
+	,ANDAND=310
+	,OROR=311
+	,NOT=312
+	,NE=313
+	,LE=314
+	,GE=315
+	,LT=316
+	,GT=317
+	,EQ=318
+	,READ=319
+	,DATA=320
+	,RESTORE=321
+	,OPEN=322
+	,CLOSE=323
+	,WINDOW=324
+	,DOT=325
+	,LINE=326
+	,CIRCLE=327
+	,TEXT=328
+	,CLEAR=329
+	,PRINTER=330
+	,WAIT=331
+	,BELL=332
+	,EOFILE=333
+	,SIN=334
+	,ASIN=335
+	,COS=336
+	,ACOS=337
+	,TAN=338
+	,ATAN=339
+	,EXP=340
+	,LOG=341
+	,SQRT=342
+	,ABS=343
+	,SGN=344
+	,INT=345
+	,FRAC=346
+	,RAN=347
+	,LEN=348
+	,VAL=349
+	,LEFT_=350
+	,RIGHT_=351
+	,MID=352
+	,STR=353
+	,CHR=354
+	,INKEY=355
+	,ASC=356
+	,DEVRD=357
+	,DEVWR=358
+	,DEVOPEN=359
+	,DEVCLOSE=360
+	,DEVCTL=361
+	,DEVSPOLL=362
+	,DEVSEEK=363
+	,DEVFLUSH=364
+	,START=365
+	,CURRENT=366
+	,MESSAGE=367
+	,M_INFO=368
+	,M_WARNING=369
+	,M_ERROR=370
+	,ADDXPLOT=371
+	,ADDYPLOT=372
+	,ADDXYPLOT=373
+	,UPDATEXPLOT=374
+	,UPDATEYPLOT=375
+	,UPDATEXYPLOT=376
+	,DELPLOT=377
+	,TITLEGRAPH=378
+	,ADDGRAPH=379
+	,DELGRAPH=380
+	,LOADGRAPH=381
+	,SAVEGRAPH=382
+	,EXPORTGRAPH=383
+	,PRINTGRAPH=384
+	,ADDPLOT=385
+	,SETPLOTDATA=386
+	,GETPLOTDATA=387
+	,SETPLOTAXES=388
+	,SETPLOTSTYLE=389
+	,SETPLOTPEN=390
+	,SETPLOTSYMBOL=391
+	,SETGRAPHTITLE=392
+	,SETGRAPHCOMMENT=393
+	,SETAXISTITLE=394
+	,SETAXISFLAGS=395
+	,SETAXISSCALE=396
+	,SETAXISSTYLE=397
+	,SETGRAPHLEGEND=398
+	,SETGRAPHCOLORS=399
+	,AUTO=400
+	,REF=401
+	,INV=402
+	,SYM=403
+	,FLT=404
+	,LEFT=405
+	,RIGHT=406
+	,TOP=407
+	,BOTTOM=408
+	,ADDDLG=409
+	,DELDLG=410
+	,HIDEDLG=411
+	,SHOWDLG=412
+	,PROGRESSDLG=413
+	,ADDWIDGET=414
+	,BUTTON=415
+	,SWITCH=416
+	,LCD=417
+	,LED=418
+	,DLGACTIVE=419
+	,INPUTDLG=420
+	,FRAME=421
+	,WHEEL=422
+	,SELECT=423
+	,GRAPH=424
+	,KNOB=425
+	,LEVEL=426
+	,SLIDER=427
+	,SLEEP=428
+	,MULTI=429
+	,OFF=430
+	,IDLE=431
+	,DEVREADY=432
+	,CURSOR=433
+	,DEFAULT=434
+	,TIMER=435
+	,DATE=436
+	,TIME=437
+	,LOWER_THAN_ELSE=438
+	,UMINUS=439
 
 
 #line 185 "/usr/local/lib/bison.h"
@@ -840,5 +963,5 @@ public:
 /* END */
 
 /* #line 236 "/usr/local/lib/bison.h" */
-#line 844 "script.bison.h"
+#line 967 "script.bison.h"
 #endif
