@@ -24,6 +24,7 @@
 #include "scriptdlg.h"
 #include <qmsgbox.h>
 #include <kapp.h>
+#include <fcntl.h>
 
 #undef None // !!!!!!!! X11/X.h defined !!!!!!!!!
 
@@ -37,19 +38,16 @@ struct array* SamplinScript::plotArrayOk(const char *astr)
    a=get_sym(astr,ARRAY,FALSE);
    
    if (a==NULL) {
-      sprintf(string,"array '%s' has not been dimed",astr);
-      error(ERROR,string);
+      error(ERROR,"array '%s' has not been dimed",astr);
       return 0;
    }
    ar=(struct array *)(a->pointer);
    if (ar->dimension!=1) {
-      sprintf(string,"array dimension must be 1 for '%s'",astr);
-      error(ERROR,string);
+      error(ERROR,"array dimension must be 1 for '%s'",astr);
       return 0;
    }
    if(ar->type!='d'){
-      sprintf(string,"Invalid array type '%s' (double required)",astr);
-      error(ERROR,string);
+      error(ERROR,"Invalid array type '%s' (double required)",astr);
       return 0;
    }
    return ar;
@@ -74,14 +72,12 @@ void SamplinScript::create_plotfnc(int fnc, char *symbol1, char *symbol2)
    if(fnc==PLOTAXES){
       a1=SamplinPlotData::axis(symbol1);
       if(a1==-1){
-	 sprintf(string,"Invalid axis %s",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid axis %s",symbol1);
 	 return;
       }
       a2=SamplinPlotData::axis(symbol2);
       if(a2==-1){
-	 sprintf(string,"Invalid axis %s",symbol2);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid axis %s",symbol2);
 	 return;
       }
    }
@@ -89,8 +85,7 @@ void SamplinScript::create_plotfnc(int fnc, char *symbol1, char *symbol2)
    if(fnc==PLOTSTYLE){
       a1=SamplinCurve::curveStyle(symbol1);
       if(a1==-1){
-	 sprintf(string,"Invalid plot style %s",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid plot style %s",symbol1);
 	 return;
       }
    }
@@ -98,22 +93,19 @@ void SamplinScript::create_plotfnc(int fnc, char *symbol1, char *symbol2)
    if(fnc==PLOTPEN){
       a1=SamplinCurve::lineStyle(symbol1);
       if(a1==-1){
-	 sprintf(string,"Invalid pen style %s",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid pen style %s",symbol1);
 	 return;
       }
    }
    if(fnc==PLOTSYMBOL){
       a1=SamplinCurve::symbolStyle(symbol1);
       if(a1==-1){
-	 sprintf(string,"Invalid symbol style %s",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid symbol style %s",symbol1);
 	 return;
       }
    }
 
-   sprintf(string,"Creating command 'plotfnc', fnc=%i",fnc);
-   error(DIAGNOSTIC,string);
+   error(DIAGNOSTIC,"Creating command 'plotfnc', fnc=%i",fnc);
    cmd=add_command(PLOTFNC);
    cmd->args=fnc;
    cmd->spointer=ar1;
@@ -194,69 +186,59 @@ void SamplinScript::plotfnc(struct command *cmd)
 
    p=pop();
    if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #2 'plot'.");
-      error(ERROR,string);
+      error(ERROR,"STRING required for argument #2 'plot'.");
    }
    name=(char *)p->pointer;
    p=pop();
    if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #1 'graph'");
-      error(ERROR,string);
+      error(ERROR,"STRING required for argument #1 'graph'");
    }
    gname=(char *)p->pointer;   
 
    s=findGraph(gname);
    if(s==NULL){
-      sprintf(string,"Cannot find graph %s.",gname);
-      error(WARNING,string);
+      error(WARNING,"Cannot find graph %s.",gname);
       return;
    }
    if(type!=PLOTADD){
       ckey=s->curveKey(name);
       if(!ckey){
-	 sprintf(string,"Cannot find plot %s in graph %s.",name,gname);
-	 error(WARNING,string);	 
+	 error(WARNING,"Cannot find plot %s in graph %s.",name,gname);	 
 	 return;
       }
    }
    
    switch(type){
     case PLOTADD:
-      sprintf(string,"Adding plot %s, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Adding plot %s, graph %s",name,gname));
       s->insertCurve(name);
       s->replot();
       break;
     case PLOTDEL:
-      sprintf(string,"Deleting plot %s, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Deleting plot %s, graph %s",name,gname));
       s->removeCurve(ckey);
       s->replot();
       break;
     case PLOTSETDATA:
     case PLOTSETDATAL:
-      sprintf(string,"Setting plot %s data, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Setting plot %s data, graph %s",name,gname));
       s->setCurveData(ckey,(double *)ar1->pointer,(double *)ar2->pointer,MIN(len,MIN(ar1->total,ar2->total)));	 
       s->replot();
       break;
     case PLOTSETDATAX:
     case PLOTSETDATAXL:
-      sprintf(string,"Setting plot %s X data, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Setting plot %s X data, graph %s",name,gname));
       s->setCurveData(ckey,(double *)ar1->pointer,from,step,MAX(0,MIN(ar1->total,len)));	 
       s->replot();
       break;
     case PLOTSETDATAY:
     case PLOTSETDATAYL:
-      sprintf(string,"Setting plot %s Y data, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Setting plot %s Y data, graph %s",name,gname));
       s->setCurveData(ckey,from,step,(double *)ar1->pointer,MAX(0,MIN(ar1->total,len)));	 
       s->replot();
       break;
     case PLOTGETDATA:
-      sprintf(string,"Getting plot %s data, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Getting plot %s data, graph %s",name,gname));
       crv=s->findCurve(ckey);
       len=MIN(crv->size(),MIN(ar1->total,ar2->total));
       for(i=0;i<len;++i){
@@ -265,21 +247,23 @@ void SamplinScript::plotfnc(struct command *cmd)
       }
       break;
     case PLOTAXES:
-      sprintf(string,"Setting plot %s axis, graph %s",name,gname);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Setting plot %s axis, graph %s",name,gname));
       s->mapCurve(ckey,cmd->arg1);
       s->mapCurve(ckey,cmd->arg2);
       s->replot();
       break;
     case PLOTSTYLE:
+      ERROR(NOTE,(NOTE,"Setting plot %s style, graph %s",name,gname));
       s->setCurveStyle(ckey,(SamplinCurveData::CurveStyle)cmd->arg1);
       s->replot();
       break;
     case PLOTPEN:
+      ERROR(NOTE,(NOTE,"Setting plot %s pen, graph %s",name,gname));
       s->setCurvePen(ckey,QPen(QColor(color1),width,(PenStyle)cmd->arg1));
       s->replot();
       break;
     case PLOTSYMBOL:
+      ERROR(NOTE,(NOTE,"Setting plot %s symbol, graph %s",name,gname));
       s->setCurveSymbol(ckey,QwtSymbol(cmd->arg1,QColor(color1),QPen(QColor(color2),width),QSize(size,size)));
       s->replot();
       break;
@@ -298,8 +282,7 @@ void SamplinScript::create_graphfnc(int fnc, int subfnc, char *symbol1, char *sy
    if(fnc==GRAPHAXISTITLE || fnc==GRAPHAXISSCALE || fnc==GRAPHAXISFLAGS || fnc==GRAPHAXISSTYLE){
       a1=SamplinPlotData::axis(symbol1);
       if(a1==-1){
-	 sprintf(string,"Invalid axis %s",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid axis %s",symbol1);
 	 return;
       }
    }
@@ -307,8 +290,7 @@ void SamplinScript::create_graphfnc(int fnc, int subfnc, char *symbol1, char *sy
    if(fnc==GRAPHAXISSTYLE){
       a2=SamplinScaleBar::style(symbol2);
       if(a2==-1){
-	 sprintf(string,"Invalid style %s",symbol2);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid style %s",symbol2);
 	 return;
       }
    }
@@ -316,14 +298,12 @@ void SamplinScript::create_graphfnc(int fnc, int subfnc, char *symbol1, char *sy
    if(fnc==GRAPHCOMMENT || fnc==GRAPHLEGEND){
       a2=SamplinPlotData::lposition(symbol2);
       if(a2==-1){
-	 sprintf(string,"Invalid legend/comment position %s",symbol2);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid legend/comment position %s",symbol2);
 	 return;
       }
    }
    
-   sprintf(string,"Creating command 'graphfnc', fnc=%i",fnc);
-   error(DIAGNOSTIC,string);
+   error(DIAGNOSTIC,"Creating command 'graphfnc', fnc=%i",fnc);
    cmd=add_command(GRAPHFNC);
    cmd->args=fnc;
    cmd->arg1=a1;
@@ -424,49 +404,42 @@ void SamplinScript::graphfnc(struct command *cmd)
    }
    p=pop();
    if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #1 'graph'");
-      error(ERROR,string);
+      error(ERROR,"STRING required for argument #1 'graph'");
    }
    gname=(char *)p->pointer;   
 
    s=findGraph(gname);
    if(s==NULL){
-      sprintf(string,"Cannot find graph %s.",gname);
-      error(WARNING,string);
+      error(WARNING,"Cannot find graph %s.",gname);
       return;
    }
    
    switch(type){
     case GRAPHAXISTITLE:
-      sprintf(string,"Setting axis %i title %s, graph %s",axis,title,gname);
-      error(NOTE,string);
+      error(NOTE,"Setting axis %i title %s, graph %s",axis,title,gname);
 //      s->setTitle(title);
 //      s->setAxisTitle(xBottom,xtitle);
       s->setAxisTitle(axis,title);
       s->replot();
       break;
     case GRAPHTITLE:
-      sprintf(string,"Setting title %s, graph %s",title,gname);
-      error(NOTE,string);      
+      error(NOTE,"Setting title %s, graph %s",title,gname);      
       s->setTitle(title);
       s->replot();
       break;
     case GRAPHCOMMENT:
-      sprintf(string,"Setting comment, graph %s",gname);
-      error(NOTE,string);      
+      error(NOTE,"Setting comment, graph %s",gname);      
       if(subfnc==1)s->setComment(title);
       s->setCommentPosition(style);
       s->replot();
       break;
     case GRAPHLEGEND:
-      sprintf(string,"Setting legend, graph %s",gname);
-      error(NOTE,string);      
+      error(NOTE,"Setting legend, graph %s",gname);      
       s->setLegendPosition(style);
       s->replot();
       break;
     case GRAPHCOLORS:
-      sprintf(string,"Setting colors, graph %s",gname);
-      error(NOTE,string);      
+      error(NOTE,"Setting colors, graph %s",gname);      
       s->setPanelColor(QColor(color));
       s->setPlotColor(QColor(color1));
       s->replot();
@@ -475,8 +448,7 @@ void SamplinScript::graphfnc(struct command *cmd)
 	{
 	   SamplinScaleBox *axisp=s->axis(axis);
 	   if(subfnc==1)prec=axisp->labelPrec();
-	   sprintf(string,"Setting axis %i style %i, prec %i",style,title,prec);
-	   error(NOTE,string);      
+	   error(NOTE,"Setting axis %i style %i, prec %i",style,title,prec);      
 	   axisp->setLabelFormat(SamplinScaleBar::Format(style),prec);
 	   s->replot();
 	}
@@ -518,6 +490,7 @@ void SamplinScript::graphfnc(struct command *cmd)
 }
 
 //===========================================
+/*
 void SamplinScript::create_addplot(int type, char *symbol1, char *symbol2)
 {
   struct command *cmd;
@@ -528,20 +501,16 @@ void SamplinScript::create_addplot(int type, char *symbol1, char *symbol2)
       a=get_sym(symbol1,ARRAY,FALSE);
       
       if (a==NULL) {
-	 sprintf(string,
-		 "array '%s' has not been dimed",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"array '%s' has not been dimed",symbol1);
 	 return;
       }
       ar1=(struct array *)(a->pointer);
       if (ar1->dimension!=1) {
-	 sprintf(string,"array dimension must be 1 for '%s'",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"array dimension must be 1 for '%s'",symbol1);
 	 return;
       }
       if(ar1->type!='d'){
-	 sprintf(string,"Invalid array type '%s' (double required)",symbol1);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid array type '%s' (double required)",symbol1);
 	 return;
       }
    }
@@ -550,21 +519,17 @@ void SamplinScript::create_addplot(int type, char *symbol1, char *symbol2)
       a=get_sym(symbol2,ARRAY,FALSE);
       
       if (a==NULL) {
-	 sprintf(string,
-		 "array '%s' has not been dimed",symbol2);
-	 error(ERROR,string);
+	 error(ERROR,"array '%s' has not been dimed",symbol2);
 	 return;
       }
       
       ar2=(struct array *)(a->pointer);
       if (ar2->dimension!=1) {
-	 sprintf(string,"array dimension must be 1 for '%s'",symbol2);
-	 error(ERROR,string);
+	 error(ERROR,"array dimension must be 1 for '%s'",symbol2);
 	 return;
       }
       if(ar2->type!='d'){
-	 sprintf(string,"Invalid array type '%s' (double required)",symbol2);
-	 error(ERROR,string);
+	 error(ERROR,"Invalid array type '%s' (double required)",symbol2);
 	 return;
       }
    }
@@ -591,48 +556,41 @@ void SamplinScript::addplot(struct command *cmd)
    
    p=pop();
    if(p->type!=STRING){
-      sprintf(string,"STRING required for argument 'color'.");
-      error(ERROR,string);
+      error(ERROR,"STRING required for argument 'color'.");
    }
    color=(char *)p->pointer;   
    
    if(type==1 || type==2){
       p=pop();
       if(p->type!=NUMBER){
-	 sprintf(string,"NUMBER required for argument 'step'.");
-	 error(ERROR,string);
+	 error(ERROR,"NUMBER required for argument 'step'.");
       }
       step=(double)p->value;
       p=pop();
       if(p->type!=NUMBER){
-	 sprintf(string,"NUMBER required for argument 'begin'.");
-	 error(ERROR,string);
+	 error(ERROR,"NUMBER required for argument 'begin'.");
       }
       begin=(double)p->value;
    }
    
    p=pop();
    if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #2 'plot'.");
-      error(ERROR,string);
+      error(ERROR,"STRING required for argument #2 'plot'.");
    }
    name=(char *)p->pointer;
    p=pop();
    if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #1 'graph'");
-      error(ERROR,string);
+      error(ERROR,"STRING required for argument #1 'graph'");
    }
    gname=(char *)p->pointer;   
    
    
-   sprintf(string,"Adding plot %s in %s",name,gname);
-   error(NOTE,string);
+   ERROR(NOTE,(NOTE,"Adding plot %s in %s",name,gname));
 
    s=findGraph(gname);
    
    if(s==NULL){
-      sprintf(string,"Cannot find graph %s.",gname);
-      error(WARNING,string);
+      error(WARNING,"Cannot find graph %s.",gname);
       return;
    }
    else{
@@ -658,7 +616,7 @@ void SamplinScript::addplot(struct command *cmd)
 
    return;
 }
-
+*/
 
 
 
@@ -765,6 +723,7 @@ GraphWindow * SamplinScript::openGraph(const char *name){
    id=newGraphId();
    gwin = new GraphWindow(NULL,name,id);
    gwin->setGeometry(10,40,590,330);
+   gwin->setRootDir(cnf.root_dir+cnf.data_dir);
    graphs.insert(id,gwin);
    connect(gwin, SIGNAL(closeSignal(uint)),this,SLOT(closeGraphSlot(uint)));
    gwin->show();
@@ -910,6 +869,7 @@ void SamplinScript::create_delgraph()
 
 }
 
+/*
 void SamplinScript::create_titlegraph()
 {
   struct command *cmd;
@@ -918,6 +878,7 @@ void SamplinScript::create_titlegraph()
   cmd=add_command(TITLEGRAPH);
 
 }
+*/
 
 void SamplinScript::create_adddlg(char *dev)
 {
@@ -970,7 +931,7 @@ void SamplinScript::create_addwidget(char *dlg, int type, char *action, char *va
    struct symbol *a;
    struct array *ar;
    
-   error(DIAGNOSTIC,"Creating command 'addwidget'");
+   ERROR(DIAGNOSTIC,(DIAGNOSTIC,"Creating command 'addwidget'"));
    cmd=add_command(ADDWIDGET);
    cmd->pointer=my_strdup(dlg);
    cmd->args=type;
@@ -980,15 +941,12 @@ void SamplinScript::create_addwidget(char *dlg, int type, char *action, char *va
       a=get_sym(action,ARRAY,FALSE);
       
       if (a==NULL) {
-	 sprintf(string,
-		 "array '%s' has not been dimed",action);
-	 error(ERROR,string);
+	 error(ERROR,"array '%s' has not been dimed",action);
 	 return;
       }
       ar=(struct array *)(a->pointer);
       if (ar->dimension!=1) {
-	 sprintf(string,"array dimension must be 1 for '%s'",action);
-	 error(ERROR,string);
+	 error(ERROR,"array dimension must be 1 for '%s'",action);
 	 return;
       }
       cmd->spointer2=ar;
@@ -1022,208 +980,6 @@ void SamplinScript::create_addwidget(char *dlg, int type, char *action, char *va
    }
 }
 
-
-void SamplinScript::create_updateplot(int type, char *symbol1, char *symbol2)
-{
-  struct command *cmd;
-  struct symbol *a;
-  struct array *ar1,*ar2;
-
-   if(symbol1!=0){
-      a=get_sym(symbol1,ARRAY,FALSE);
-      
-      if (a==NULL) {
-	 sprintf(string,
-		 "array '%s' has not been dimed",symbol1);
-	 error(ERROR,string);
-	 return;
-      }
-      ar1=(struct array *)(a->pointer);
-      if (ar1->dimension!=1) {
-	 sprintf(string,"array dimension must be 1 for '%s'",symbol1);
-	 error(ERROR,string);
-	 return;
-      }
-      if(ar1->type!='d'){
-	 sprintf(string,"Invalid array type '%s' (double required)",symbol1);
-	 error(ERROR,string);
-	 return;
-      }
-   }
-   
-   if(symbol2!=0){
-      a=get_sym(symbol2,ARRAY,FALSE);
-      
-      if (a==NULL) {
-	 sprintf(string,
-		 "array '%s' has not been dimed",symbol2);
-	 error(ERROR,string);
-	 return;
-      }
-      
-      ar2=(struct array *)(a->pointer);
-      if (ar2->dimension!=1) {
-	 sprintf(string,"array dimension must be 1 for '%s'",symbol2);
-	 error(ERROR,string);
-	 return;
-      }
-      if(ar2->type!='d'){
-	 sprintf(string,"Invalid array type '%s' (double required)",symbol2);
-	 error(ERROR,string);
-	 return;
-      }
-   }
-   
-   error(DIAGNOSTIC,"Creating command 'updateplot'");
-   cmd=add_command(UPDATEPLOT);
-   cmd->args=type;
-   cmd->spointer=ar1;
-   cmd->spointer2=ar2;
-}
-
-void SamplinScript::create_delplot()
-{
-  struct command *cmd;
- 
-   error(DIAGNOSTIC,"Creating command 'delplot'");
-   cmd=add_command(DELPLOT);
-}
-
-
-void SamplinScript::updateplot(struct command *cmd)
-{
-   GraphPanel *s;
-   char *name,*gname,*color;
-   struct stackentry *p;
-   struct array *ar1,*ar2;
-   float begin, step,*tmp;
-   int i,type;
-   
-   ar1=(struct array *)cmd->spointer;
-   ar2=(struct array *)cmd->spointer2;
-   type=cmd->args;
-   
-   p=pop();
-   if(p->type!=STRING){
-      sprintf(string,"STRING required for argument 'color'.");
-      error(ERROR,string);
-   }
-   color=(char *)p->pointer;   
-
-   if(type==1 || type==2){
-      p=pop();
-      if(p->type!=NUMBER){
-	 sprintf(string,"NUMBER required for argument 'step'.");
-	 error(ERROR,string);
-      }
-      step=(double)p->value;
-      p=pop();
-      if(p->type!=NUMBER){
-	 sprintf(string,"NUMBER required for argument 'begin'.");
-	 error(ERROR,string);
-      }
-      begin=(double)p->value;
-   }
-   p=pop();
-   if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #2 'plot'.");
-      error(ERROR,string);
-   }
-   name=(char *)p->pointer;
-   p=pop();
-   if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #1 'graph'.");
-      error(ERROR,string);
-   }
-   gname=(char *)p->pointer;   
-   
-   
-   sprintf(string,"Updating plot %s in %s",name,gname);
-   error(NOTE,string);
-
-   s=findGraph(gname);
-
-   if(s==NULL){
-      sprintf(string,"Cannot find graph %s.",gname);
-      error(WARNING,string);
-      return;
-   }
-   else{
-      uint ckey;
-      
-      ckey=s->curveKey(name);
-      if(ckey){
-	 s->setCurvePen(ckey,QPen(QColor(color),1,SolidLine));
-	 switch(type){
-	  case 1:
-	    s->setCurveData(ckey,begin,step,(double *)ar1->pointer,ar1->total);
-	    break;
-	  case 2:
-	    s->setCurveData(ckey,(double *)ar1->pointer,begin,step,ar1->total);	 
-	    break;
-	  case 3:
-	    s->setCurveData(ckey,(double *)ar1->pointer,(double *)ar2->pointer,MIN(ar1->total,ar2->total));	 
-	    break;
-	 }
-	 s->replot();
-      }
-      else{
-	 sprintf(string,"Cannot find plot %s in graph %s.",name,gname);
-	 error(WARNING,string);	 
-      }
-   
-   }
-   return;
-}
-
-
-void SamplinScript::delplot(struct command *cmd)
-{
-   GraphPanel *s;
-   char *gname,*name;
-   struct stackentry *p;
-   int i;
-   
-   p=pop();
-   if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #2.");
-      error(ERROR,string);
-   }
-   name=(char *)p->pointer;
-   
-   p=pop();
-   if(p->type!=STRING){
-      sprintf(string,"STRING required for argument #1.");
-      error(ERROR,string);
-   }
-   gname=(char *)p->pointer;
-   
-   sprintf(string,"Deleting plot %s in %s",name,gname);
-   error(NOTE,string);
-   
-   s=findGraph(gname);
-   
-   if(s==NULL){
-      sprintf(string,"Cannot find graph %s.",gname);
-      error(WARNING,string);
-      return;
-   }
-
-   uint ckey;
-   
-   ckey=s->curveKey(name);
-   if(ckey){
-      s->removeCurve(ckey);
-      s->replot();
-   }
-   else{
-      sprintf(string,"Cannot find plot %s in graph %s.",name,gname);
-      error(WARNING,string);	 
-   }
-
-   return;
-}
-
 void SamplinScript::loadgraph(struct command *cmd)
 {
    char *name,*file;
@@ -1235,26 +991,22 @@ void SamplinScript::loadgraph(struct command *cmd)
      {
 	p=pop();
 	file=(char *)(p->pointer);
-	sprintf(string,"Loading graph '%s' from file '%s'",name,file);
+	error(NOTE,"Loading graph '%s' from file '%s'",name,file);
      }
    else {
       file=NULL;
-      sprintf(string,"Loading graph '%s'",name);
+      error(NOTE,"Loading graph '%s'",name);
    }
    p=pop();
    name=(char *)(p->pointer);
 
-   error(NOTE,string);
-
    s=findGraph(name);
    if(s==NULL){
-      sprintf(string,"Cannot find graph '%s'.",name);
-      error(WARNING,string);      
+      error(WARNING,"Cannot find graph '%s'.",name);      
    }
    else if(s->load(file)==FALSE)
      {
-	sprintf(string,"Cannot open file '%s' for reading.",file);
-	error(WARNING,string);      	
+	error(WARNING,"Cannot open file '%s' for reading.",file);      	
      }
    
    return;
@@ -1271,28 +1023,24 @@ void SamplinScript::savegraph(struct command *cmd)
      {
 	p=pop();
 	file=(char *)(p->pointer);
-	sprintf(string,"Saving graph '%s' to file '%s'",name,file);
+	error(NOTE,"Saving graph '%s' to file '%s'",name,file);
      }
    else
      {
 	file=NULL; 
-	sprintf(string,"Saving graph '%s'",name);
+	error(NOTE,"Saving graph '%s'",name);
      }
    
    p=pop();
    name=(char *)(p->pointer);
    
-   error(NOTE,string);
-
    s=findGraph(name);
    if(s==NULL){
-      sprintf(string,"Cannot find graph '%s'.",name);
-      error(WARNING,string);      
+      error(WARNING,"Cannot find graph '%s'.",name);      
    }
    else if(s->save(file)==FALSE)
      {
-	sprintf(string,"Cannot open file '%s' for writing.",file);
-	error(WARNING,string);      	
+	error(WARNING,"Cannot open file '%s' for writing.",file);      	
      }
    
    return;
@@ -1309,28 +1057,24 @@ void SamplinScript::exportgraph(struct command *cmd)
      {
 	p=pop();
 	file=(char *)(p->pointer);
-	sprintf(string,"Exporting graph '%s' to file '%s'",name,file);
+	error(NOTE,"Exporting graph '%s' to file '%s'",name,file);
      }
    else
      {
 	file=NULL; 
-	sprintf(string,"Exporting graph '%s'",name);
+	error(NOTE,"Exporting graph '%s'",name);
      }
    
    p=pop();
    name=(char *)(p->pointer);
    
-   error(NOTE,string);
-
    s=findGraph(name);
    if(s==NULL){
-      sprintf(string,"Cannot find graph '%s'.",name);
-      error(WARNING,string);      
+      error(WARNING,"Cannot find graph '%s'.",name);      
    }
    else if(s->export(file)==FALSE)
      {
-	sprintf(string,"Cannot open file '%s' for writing.",file);
-	error(WARNING,string);      	
+	error(WARNING,"Cannot open file '%s' for writing.",file);      	
      }
    
    return;
@@ -1346,13 +1090,11 @@ void SamplinScript::printgraph(struct command *cmd)
    p=pop();
    name=(char *)(p->pointer);
    
-   sprintf(string,"Printing graph '%s'",name);
-   error(NOTE,string);
+   error(NOTE,"Printing graph '%s'",name);
 
    s=findGraph(name);
    if(s==NULL){
-      sprintf(string,"Cannot find graph '%s'.",name);
-      error(WARNING,string);      
+      error(WARNING,"Cannot find graph '%s'.",name);      
    }
 
    s->print();
@@ -1371,12 +1113,10 @@ void SamplinScript::addgraph(struct command *cmd)
    
    name=(char *)(p->pointer);
    
-   sprintf(string,"Adding graph window '%s'",name);
-   error(NOTE,string);
+   error(NOTE,"Adding graph window '%s'",name);
 
    if((gwin=findGraphWindow(name))!=NULL){
-      sprintf(string,"Graph window '%s' already open.",name);
-      error(WARNING,string);
+      error(WARNING,"Graph window '%s' already open.",name);
    }
    else  openGraph(name);
    
@@ -1394,57 +1134,18 @@ void SamplinScript::delgraph(struct command *cmd)
    
    name=(char *)(p->pointer);
    
-   sprintf(string,"Deleting graph '%s'",name);
-   error(NOTE,string);
-
+   error(NOTE,"Deleting graph '%s'",name);
    
    s=findGraphWindow(name);
    
    if(s==NULL){
-      sprintf(string,"Cannot find graph '%s'.",name);
-      error(WARNING,string);      
+      error(WARNING,"Cannot find graph '%s'.",name);      
    }
    else closeGraph(name);
    
    return;
 }
 
-void SamplinScript::titlegraph(struct command *cmd)
-{
-   char *name, *title,*xtitle,*ytitle;
-   struct stackentry *p;
-   int status;
-   GraphPanel *s;
-   
-   p=pop();
-   ytitle=(char *)(p->pointer);   
-   p=pop();
-   xtitle=(char *)(p->pointer);   
-   p=pop();
-   title=(char *)(p->pointer);
-   p=pop();
-   name=(char *)(p->pointer);
-   
-   sprintf(string,"Title graph '%s'",name);
-   error(NOTE,string);
-
-
-   s=findGraph(name);
-
-   if(s==NULL){
-      sprintf(string,"Cannot find graph '%s'.",name);
-      error(WARNING,string);      
-   }
-   else {
-      s->setTitle(title);
-      s->setAxisTitle(xBottom,xtitle);
-      s->setAxisTitle(yLeft,ytitle);
-      s->replot();
-   }
-   
-   return;
-}
-
 void SamplinScript::progressdlg(struct command *cmd) 
 {
    struct stackentry *p;
@@ -1469,11 +1170,9 @@ void SamplinScript::progressdlg(struct command *cmd)
 	pd->setMinimumDuration(0);
 	pd -> setProgress(0);
 	get_sym("progress_cancel",NUMBER,TRUE)->value=FALSE;   
-	task_cnt=0;
      }
    if(cmd->args==1){
 	pd -> setProgress(steps);
-	task_cnt=0;      
    }
    
 }
@@ -1492,8 +1191,7 @@ void SamplinScript::adddlg(struct command *cmd)
    title=my_strdup((char *)p->pointer);
    name=(char *)cmd->pointer;
 
-   sprintf(string,"Adding dialog: %s",name);
-   error(NOTE,string);
+   ERROR(NOTE,(NOTE,"Adding dialog: %s",name));
    
    for(dlg=dialogs.first();dlg!=0;dlg=dialogs.next()){
       if(!strcmp(name,dlg->name) && strlen(name)==strlen(dlg->name)){
@@ -1521,8 +1219,7 @@ void SamplinScript::hidedlg(struct command *cmd)
    found=FALSE;
    name=(char *)cmd->pointer;
 
-   sprintf(string,"Hiding dialog: %s",name);
-   error(NOTE,string);   
+   ERROR(NOTE,(NOTE,"Hiding dialog: %s",name));   
    
    for(dlg=dialogs.first();dlg!=0;dlg=dialogs.next()){
       if(!strcmp(name,dlg->name) && strlen(name)==strlen(dlg->name)){
@@ -1538,8 +1235,7 @@ void SamplinScript::hidedlg(struct command *cmd)
       }
    }
    if(found==FALSE){
-      sprintf(string,"Cannot find dialog %s.",name);	 
-      error(WARNING,string);   
+      error(WARNING,"Cannot find dialog %s.",name);   
    }
    
    return;
@@ -1554,8 +1250,7 @@ void SamplinScript::deldlg(struct command *cmd)
    found=FALSE;
    name=(char *)cmd->pointer;
 
-   sprintf(string,"Deleting dialog: %s",name);
-   error(NOTE,string);   
+   ERROR(NOTE,(NOTE,"Deleting dialog: %s",name));   
    
    for(dlg=dialogs.first();dlg!=0;dlg=dialogs.next()){
       if(!strcmp(name,dlg->name) && strlen(name)==strlen(dlg->name)){
@@ -1571,8 +1266,7 @@ void SamplinScript::deldlg(struct command *cmd)
       }
    }
    if(found==FALSE){
-      sprintf(string,"Cannot find dialog %s.",name);	 
-      error(WARNING,string);   
+      error(WARNING,"Cannot find dialog %s.",name);   
    }
    
    return;
@@ -1590,8 +1284,7 @@ void SamplinScript::showdlg(struct command *cmd)
    found=FALSE;
    name=(char *)cmd->pointer;
 
-   sprintf(string,"Show dialog: %s",name);
-   error(NOTE,string);      
+   ERROR(NOTE,(NOTE,"Show dialog: %s",name));      
    
    for(dlg=dialogs.first();dlg!=0;dlg=dialogs.next()){
       if(!strcmp(name,dlg->name) && strlen(name)==strlen(dlg->name)){
@@ -1684,6 +1377,7 @@ void SamplinScript::showdlg(struct command *cmd)
 		    GraphPanel *tmpPtr;
 		    tmpPtr = new GraphPanel(dlg->dlg,"graph");
 		    tmpPtr->setGeometry(wdt->x,wdt->y,wdt->w, wdt->h);	       
+		    tmpPtr->setRootDir(cnf.root_dir+cnf.data_dir);
 		    wdt->pointer=tmpPtr;
 		    break;	       
 		 }	       
@@ -1840,8 +1534,7 @@ void SamplinScript::showdlg(struct command *cmd)
       }
    }
    if(found==FALSE){
-      sprintf(string,"Cannot find dialog %s.",name);	 
-      error(WARNING,string);   
+      error(WARNING,"Cannot find dialog %s.",name);   
    }
 
    return;
@@ -1908,8 +1601,7 @@ void SamplinScript::addwidget(struct command *cmd)
 
 	    ar=(struct array *)current->spointer2;
 	    if(ar->type!='s'){
-	       sprintf(string,"Invalid array type (string required)");
-	       error(ERROR,string);
+	       error(ERROR,"Invalid array type (string required)");
 	       return;
 	    }
 	    wdt->count=ar->total;
@@ -2002,100 +1694,74 @@ void SamplinScript::addwidget(struct command *cmd)
       } 
    }
    if(found==FALSE){
-      sprintf(string,"Cannot find dialog %s.",name);	 
-      error(WARNING,string);   
+      error(WARNING,"Cannot find dialog %s.",name);   
    }
    
    return;
 }
 
-void SamplinScript::create_myread(char type) /* create command 'read' */
+void SamplinScript::create_myreaddlg(char type) /* create command 'read' */
 {
   struct command *cmd;
 
-  error(DIAGNOSTIC,"Creating command 'read'");
-  cmd=add_command(MYREAD);
+  error(DIAGNOSTIC,"Creating command 'readdlg'");
+  cmd=add_command(MYREADDLG);
   lastcommand=cmd;
   cmd->tag=type;
   cmd->args=FALSE;
 }
 
-
-void SamplinScript::myread(struct command *cmd) /* read string or double */
+void SamplinScript::myreaddlg(struct command *cmd) /* read string or double */
 {
    Script_input *dlg;
    double d;
-  int i,c;
-  static char buffer[1000];
-  struct stackentry *s,*p;
-  FILE *str;
-  int tileol; /* read til end of line */
+   int i,c;
+   static char buffer[1000];
+   struct stackentry *s,*p;
+   FILE *str;
+   int tileol; /* read til end of line */
 
+   ERROR(NOTE,(NOTE,"begin myreaddlg"));
    
    p=pop();
+   //   if(p->type!=PTR)error(ERROR,"Input expected pointer on stack");
    
    tileol=cmd->args;
-   if (DIAGNOSTIC<=infolevel) {
-      sprintf(string,"Waiting for input til %s",tileol ? "end of line":"whitespace");
-      error(DIAGNOSTIC,string);
+   
+   ERROR(DIAGNOSTIC,(DIAGNOSTIC,"Waiting for input til %s",tileol ? "end of line":"whitespace"));
+   
+   dlg = new Script_input(NULL,"Input dialog");
+   dlg->setTitle(inputprompt);
+   strcpy(buffer,"");
+   if(cmd->tag=='d')
+     sprintf(buffer,"%g",*(double*)(p->pointer));
+   if(cmd->tag=='s')
+     strncpy(buffer,(char *)p->pointer,999);
+   
+   dlg->setText(buffer);
+   dlg->show();
+   strncpy(buffer,dlg->getText(),999);
+   delete dlg;	
+   
+   if (cmd->tag=='s') { /* read string */
+      s=push();
+      s->type=STRING;
+      s->pointer=my_strdup(buffer);
+      //     if(p->pointer)free(p->pointer);
+//     p->pointer=my_strdup(buffer);
    }
-
-   
-   
-  if (currentstream==NULL) {
-     if(msgwin==NULL){
-	str=stdin;
-	printf("%s",inputprompt);
-     }
-     else{
-	dlg = new Script_input(NULL,"Input dialog");
-	dlg->setTitle(inputprompt);
-	strcpy(buffer,"");
-	if(p->type==NUMBER)
-	  sprintf(buffer,"%g",p->value);
-	if(p->type==STRING)
-	  strncpy(buffer,(char *)p->pointer,999);
-	
-	dlg->setText(buffer);
-	dlg->show();
-	strncpy(buffer,dlg->getText(),999);
-	delete dlg;	
-     }
-  }
-     
-  else {
-    str=currentstream;
-    fflush(currentstream);
-  }
-
-   if(currentstream!=NULL || msgwin==NULL){
-      i=0;
-      while (isspace(c=fgetc(str)) && c!='\n');  /* skip whitespace */
-      if (c!='\n' && c!=EOF) {
-	 do {
-	    *(buffer+i)=c;
-	    i++;
-	    c=fgetc(str);
-	 } while (!(isspace(c) && !tileol) &&
-		  c!='\n' && i<1000 && c!=EOF);
-      }
-      *(buffer+i)='\0';
+   else { /* read double */
+      //    s=push();
+      //    s->type=NUMBER;
+      if (/*c==EOF ||*/ (sscanf(buffer,"%lf",&d)==0))
+	//      s->value=0.0;
+	*(double*)(p->pointer)=0.0;
+      else 
+	//      s->value=d;
+	*(double*)(p->pointer)=d;
    }
-
-   strcpy(inputprompt,(c=='\n')?"?":"");
    
-  if (cmd->tag=='s') { /* read string */
-    s=push();
-    s->type=STRING;
-    s->pointer=my_strdup(buffer);}
-  else { /* read double */
-    s=push();
-    s->type=NUMBER;
-    if (c==EOF || (sscanf(buffer,"%lf",&d)==0))
-      s->value=0.0;
-    else 
-      s->value=d;
-  }
+   ERROR(NOTE,(NOTE, "end myreaddlg"));
 }
 
 void SamplinScript::create_cursor(int f)
@@ -2133,8 +1799,7 @@ void SamplinScript::cursor(struct command *cmd)
 
 void SamplinScript::widgetSlot(widgetitem *w)
 {
-   sprintf(string,"Message got: type:%i, active=%i,action=%s\n",w->type,w->active,w->action);
-   error(NOTE,string);
+   ERROR(NOTE,(NOTE,"Message got: type:%i, active=%i,action=%s\n",w->type,w->active,w->action));
    
    if(waitMulti==NULL)waitMulti=w;
    
@@ -2147,8 +1812,7 @@ void SamplinScript::widgetAction(void)
    if(multiflag==FALSE)return;
    
    if(waitMulti!=NULL){
-      sprintf(string,"Message exec: type:%i, active=%i,action=%s\n",waitMulti->type,waitMulti->active,waitMulti->action);
-      error(NOTE,string);
+      ERROR(NOTE,(NOTE,"Message exec: type:%i, active=%i,action=%s\n",waitMulti->type,waitMulti->active,waitMulti->action));
       if(waitMulti->active==FALSE){
 	 waitMulti->active=TRUE;
 	 jmpcmd.type=IGOSUB;
@@ -2167,11 +1831,3 @@ void SamplinScript::progressSlot()
    get_sym("progress_cancel",NUMBER,TRUE)->value=TRUE;   
 }
 
-void SamplinScript::crashHandler(void)
-{
-
-   pd->setTotalSteps(1000);
-   pd -> setProgress(0);
-   usleep(100000);
-   pd->reset();      
-}
