@@ -48,7 +48,7 @@ SamplinDevice::SamplinDevice( void )
    ntimeout=10;
    
    gpib_address=0;
-   gpib_eos=0x0d;
+   eos=0x0d;
    gpib_master=FALSE;
    gpib_dcl=FALSE;
    gpib_llo=FALSE;
@@ -78,7 +78,7 @@ SamplinDevice &SamplinDevice::operator=(SamplinDevice &d)
    serial_speed=d.getSpeed();
    ntimeout=d.getNTimeout();
    gpib_address=d.getAddress();
-   gpib_eos=d.getEos();
+   eos=d.getEos();
    gpib_master=d.getMaster();
 
    return(*this);
@@ -139,16 +139,28 @@ int SamplinDevice::Write(const char *data, const int len, const int binary)
     case DEVICE_GPIB:
       if(host.isEmpty()){
 //	 stat = ibwrt(handle,data,n);
-	 stat=ibFunc(handle,gpib_master ? IBWRT:DVWRT,gpib_address,data,n);	 
+	 stat=ibFunc(handle,IBEOS,eos);	 
 	 if(stat & ERR)ret=-2;
+	 else {
+	    stat=ibFunc(handle,gpib_master ? IBWRT:DVWRT,gpib_address,data,n);	 
+	    if(stat & ERR)ret=-2;
+	 }
       }
       else{
-	 stat=ibFuncRemote(rhandle,handle,gpib_master ? IBWRT:DVWRT,gpib_address,data,n);
+	 stat=ibFuncRemote(rhandle,handle,IBEOS,eos,0,0);
 	 if(stat == ERR){
 	    ret=-3;
 	    errlog=clnt_sperror(rhandle,FAIL_MSG);
 	 }
 	 else if(stat & ERR)ret=-2;
+	 else{
+	    stat=ibFuncRemote(rhandle,handle,gpib_master ? IBWRT:DVWRT,gpib_address,data,n);
+	    if(stat == ERR){
+	       ret=-3;
+	       errlog=clnt_sperror(rhandle,FAIL_MSG);
+	    }
+	    else if(stat & ERR)ret=-2;
+	 }
 	    
       }
       if(ret==0)usleep(delay);
@@ -156,12 +168,20 @@ int SamplinDevice::Write(const char *data, const int len, const int binary)
 
     case DEVICE_SERIAL:
     case DEVICE_VOID:
-      mybuff=(char *)calloc(n+4+1,sizeof(char));
-      memset(mybuff,0,n+4+1);
+      mybuff=(char *)calloc(n+4+2,sizeof(char));
+      memset(mybuff,0,n+4+2);
       memcpy(mybuff,data,n);
       if(!binary){
-	 *(mybuff+n)='\r';
-	 ++n;
+//	 *(mybuff+n)='\r'; !!! EOS
+	 if(eos & 0xff00){
+	    *(mybuff+n)=(eos & 0xff00)>>8;
+	    *(mybuff+n+1)=(eos & 0xff);
+	    n+=2;
+	 }
+	 else {
+	    *(mybuff+n)=eos;
+	    ++n;
+	 }
       }
       
       if(host.isEmpty()){
@@ -181,13 +201,13 @@ int SamplinDevice::Write(const char *data, const int len, const int binary)
 	 req.buffer.buffer_val=mybuff;
 	 req.buffer.buffer_len=n;
 	 result=rserial_dorequest_1(&req,rhandle,ntimeout);
+	 ret=0;
 	 if(result==NULL){
 	    errlog=clnt_sperror(rhandle,FAIL_MSG);
 	    ret=-3;
 	 }
-	 else{
-	    ret=0;
-	 }
+	 else if(result->ret==-1)ret=-2;
+
       }
       if(delay!=0)usleep(delay);
       free(mybuff);
@@ -196,13 +216,25 @@ int SamplinDevice::Write(const char *data, const int len, const int binary)
    return(ret);
 }
 
-int SamplinDevice::Ioctl(int request, long arg)
+int SamplinDevice::Flush(void)
+{
+   return 0;  
+}
+int SamplinDevice::Seek(long pos, int whence)
+{
+   return 0;
+}
+
+// support for long *arg added, warning highly explosive !
+// default len=sizeof(long) to minimize danger
+
+int SamplinDevice::Ioctl(int request, char *data, const int len)
 {
    int ret;
    serial_request req, *result;
-   char mybuff[sizeof(long)+4];
+//   char mybuff[sizeof(long)+4];
    
-   *(long*)mybuff=arg;
+//   *(long*)mybuff=arg;
    ret=0;
    if(handle==0)return(-1);
 
@@ -213,7 +245,7 @@ int SamplinDevice::Ioctl(int request, long arg)
     case DEVICE_SERIAL:
     case DEVICE_VOID:
       if(host.isEmpty()){
-	 ret=ioctl(handle,request,mybuff);
+	 ret=ioctl(handle,request,/*mybuff*/ data);
 	 if(ret==-1)--ret;
       }
       else{
@@ -224,8 +256,8 @@ int SamplinDevice::Ioctl(int request, long arg)
 	 req.request=REQ_IOCTL;
 	 req.handle=handle;
 	 req.arg=request;
-	 req.buffer.buffer_val=mybuff;
-	 req.buffer.buffer_len=sizeof(long);
+	 req.buffer.buffer_val=data; //mybuff;
+	 req.buffer.buffer_len=len; //sizeof(long);
 	 result=rserial_dorequest_1(&req,rhandle,ntimeout);
 	 if(result==NULL){
 	    errlog=clnt_sperror(rhandle,FAIL_MSG);
@@ -233,6 +265,8 @@ int SamplinDevice::Ioctl(int request, long arg)
 	 }
 	 else{
 	    ret=0;
+	    memcpy(data,result->buffer.buffer_val,result->buffer.buffer_len);
+	    free(result->buffer.buffer_val);
 	 }
       }
       break;
@@ -246,6 +280,10 @@ int SamplinDevice::Read(char *data, const int len, const int binary)
    serial_request req, *result;
    int i, retry_count;
    char chout;
+   int eos_flag,eos1,eos2;
+   
+   eos1=(eos & 0xff);
+   eos2=(eos & 0xff00)>>8;
    
    ret=0;
 
@@ -257,16 +295,28 @@ int SamplinDevice::Read(char *data, const int len, const int binary)
       if(host.isEmpty()){
 //	 stat = ibrd(handle,data,len);
 //	 if(stat & ERR)ret=-2;
-	 stat=ibFunc(handle,gpib_master ? IBRD:DVRD,gpib_address,data,len);
+	 stat=ibFunc(handle,IBEOS,eos);
 	 if(stat & ERR)ret=-2;	 
+	 else{
+	    stat=ibFunc(handle,gpib_master ? IBRD:DVRD,gpib_address,data,len);
+	    if(stat & ERR)ret=-2;	 
+	 }
       }
       else{
-	 stat=ibFuncRemote(rhandle,handle,gpib_master ? IBRD:DVRD,gpib_address,data,len);
+	 stat=ibFuncRemote(rhandle,handle,IBEOS,eos,0,0);
 	 if(stat == ERR){
 	    ret=-3;
 	    errlog=clnt_sperror(rhandle,FAIL_MSG);
 	 }
 	 else if(stat & ERR)ret=-2;
+	 else{
+	    stat=ibFuncRemote(rhandle,handle,gpib_master ? IBRD:DVRD,gpib_address,data,len);
+	    if(stat == ERR){
+	       ret=-3;
+	       errlog=clnt_sperror(rhandle,FAIL_MSG);
+	    }
+	    else if(stat & ERR)ret=-2;
+	 }
       }
       if(ret==0)usleep(delay);
       break;
@@ -277,24 +327,32 @@ int SamplinDevice::Read(char *data, const int len, const int binary)
 	 memset(data,0,len);
 	 i=0;
 
-	 if(type==DEVICE_VOID){
+	 /*	 if(type==DEVICE_VOID){
 	    ret=read(handle, data, len);
 	    if(ret!=-1)i=ret;
 	 }
-	 else{
-	 
+	  else{*/
+	 eos_flag=0;
+	 do{
+	    retry_count=0;
 	    do{
-	       retry_count=0;
-	       do{
-		  ret=read(handle, &chout, sizeof(chout));
-		  ++retry_count;
-	       }while(ret==-1 && errno==11 && retry_count<RETRY_READ);
-	       
-	       if(ret!=-1 && (chout!='\n' || binary)){
-		  *(data+i)=chout;
-		  ++i;
-	       }
-	    }while(ret!=0 && ret!=-1 && (chout!='\n' || binary) && i<len);
+	       ret=read(handle, &chout, sizeof(chout));
+	       ++retry_count;
+	    }while(ret==-1 && errno==11 && retry_count<RETRY_READ);
+	    
+	    if(ret!=-1 && (chout!=eos1 /*'\n'*/ || binary )){
+	       *(data+i)=chout;
+	       ++i;
+	    }
+
+	    if(i>0)
+	      if(eos1==chout && (eos2 == *(data+i-1) || !eos2))
+		eos_flag=1;
+	    
+	 }while(ret!=0 && ret!=-1 && ((!eos_flag && chout!=0)/*'\n'*/ || binary) && i<len);
+
+	 if(!binary && eos_flag && eos2){
+	    *(data+i-1)=0;
 	 }
 
 	 if(ret!=-1)ret=i;
@@ -305,10 +363,11 @@ int SamplinDevice::Read(char *data, const int len, const int binary)
 	 req.device="";
 	 req.client="";
 	 req.count=len;
-//	 printf("req len=%i\n",len);
+	 req.ret=eos;
+	 //	 printf("req len=%i\n",len);
 	 if(type==DEVICE_VOID)req.arg=0;
 	 else req.arg=RETRY_READ;
-	   
+	 
 	 if(binary==0)req.request=REQ_READ_STR;
 	 else req.request=REQ_READ_BIN;
 	 req.handle=handle;
@@ -350,6 +409,7 @@ int SamplinDevice::Ready( void )
     case DEVICE_SERIAL:
       if(host.isEmpty()){
 	 maxfd=handle+1;
+	 FD_ZERO(&readfs);
 	 FD_SET(handle,&readfs);
 	 timeout.tv_usec=0;
 	 timeout.tv_sec=0;
@@ -733,7 +793,7 @@ QDataStream &operator<<( QDataStream &s, SamplinDevice &dev )
    s << dev.getParity();
    s << dev.getTimeout();   
    s << dev.getNTimeout();
-   s << (int)255;
+   s << (int)254;
    s << dev.getAddress();
    s << dev.getEos();
    s << dev.getMaster();
@@ -792,6 +852,17 @@ char ch;
 	 dev.setMaster(ch);
 	 s >> i;
       }
+      else if(i==254){
+	 s >> i;
+	 dev.setAddress(i);
+	 s >> i;
+	 dev.setEos(i);
+	 s >> ch;
+	 dev.setMaster(ch);
+	 s >> i;
+      }
+
+   
    }
 
    return(s);
