@@ -28,36 +28,39 @@
 #include <qbitmap.h>
 #include <qdrawutl.h>
 #include <qfiledlg.h>
+#include <kfiledialog.h>
 #include <qmessagebox.h>
 
 #include "GraphPanel.moc"
 
-#include <samplin.h>
+//#include <samplin.h>
+#include <sys/param.h>
 
 #define LIST_WIDTH 160
 #define HANDLE_WIDTH 16
+#define VISIBLE_ID 1
 
 #define SERIES_BEGIN 5
 
-#define OLD_GRAPH_EXT ".1.SampLin"
-#define GRAPH_EXT ".Graph"
-#define EXPORT_TXT_EXT ".txt"
-#define EXPORT_PIC_EXT ".pic"
-#define EXPORT_BMP_EXT ".bmp"
+#include <kapp.h>
+#include <kclipboard.h>
 
+//#include <qdragobject.h>
 
 SamplinCurve *clipboard_curve=0;
 
 void PropItem::paint( QPainter *p )
 {
-   p->drawPixmap( 2, 1, *pm );
-   QFontMetrics fm = p->fontMetrics();
-   int yPos;                       // vertical text position
-   if ( pm->height() < fm.height() )
-     yPos = 1+fm.ascent() + fm.leading()/2;
-   else
-     yPos = 1+pm->height()/2 - fm.height()/2 + fm.ascent();
-   p->drawText( pm->width() + 10, yPos, text() );
+   if(pm!=0){
+      p->drawPixmap( 2, 1, *pm );
+      QFontMetrics fm = p->fontMetrics();
+      int yPos;                       // vertical text position
+      if ( pm->height() < fm.height() )
+	yPos = 1+fm.ascent() + fm.leading()/2;
+      else
+	yPos = 1+pm->height()/2 - fm.height()/2 + fm.ascent();
+      p->drawText( pm->width() + 10, yPos, text() );
+   }
 }
 
 int PropItem::height(const PropBox *lb ) const
@@ -98,7 +101,8 @@ GraphPanel::GraphPanel ( QWidget* parent, const char* name )
 :QFrame( parent, name )
 {
    spath=NULL;
-   grseries=NULL;
+   //grseries=NULL;
+   grcurve=NULL;
    groptions=NULL;
    graxis=NULL;
    
@@ -116,10 +120,12 @@ GraphPanel::GraphPanel ( QWidget* parent, const char* name )
 //   graph->setAxisTitle(SamplinPlot::xBottom, "X axis");
 //   graph->setAxisTitle(SamplinPlot::yLeft, "Y axis");   
    
-   connect(graph, SIGNAL(objClick(const QMouseEvent *,int,uint)),
-	   SLOT(graphClick(const QMouseEvent *,int,uint)));
+   connect(graph, SIGNAL(objClick(int,uint, int, double, double)),
+	   SLOT(graphClick(int,uint, int,double, double)));
    connect(graph, SIGNAL(objDoubleClick(const QMouseEvent *,int,uint)),
 	   SLOT(graphDoubleClick(const QMouseEvent *,int,uint)));   
+   connect(graph, SIGNAL(filenameChanged()),
+	   SLOT(filenameChangedSlot()));   
    
    list = new GraphPropBox(this,"Series");
 /*
@@ -146,10 +152,14 @@ GraphPanel::GraphPanel ( QWidget* parent, const char* name )
 
    handlebutton= new QPushButton(this);
    connect( handlebutton, SIGNAL( pressed() ), this, SLOT( handleClick() ));
-  
+
+   popimport = new KPopupMenu("Import");
+   CHECK_PTR(popimport);
+   popimport->insertItem("&ASCII data",this,SLOT(importASCII()));
+   
    popexport = new KPopupMenu("Export");
    CHECK_PTR(popexport);
-   popexport->insertItem("&Text data",this,SLOT(exportData()));
+   popexport->insertItem("&ASCII data",this,SLOT(exportASCII()));
    popexport->insertItem("&Qt picture",this,SLOT(exportPic()));
    popexport->insertItem("&BMP bitmap",this,SLOT(exportBMP()));
 
@@ -159,8 +169,12 @@ GraphPanel::GraphPanel ( QWidget* parent, const char* name )
    popgraph->insertItem("&Open",this,SLOT(load()));
    popgraph->insertItem("&Save",this,SLOT(save()));
    popgraph->insertItem("Save &as",this,SLOT(saveAs()));
+   popgraph->insertItem("&Import",popimport);
    popgraph->insertItem("&Export",popexport);
-   popgraph->insertItem("&Print",this,SLOT(print()));
+   popgraph->insertItem("P&rint",this,SLOT(print()));
+   popgraph->insertSeparator();
+   popgraph->insertItem("&New",this,SLOT(editNew()));
+   popgraph->insertItem("&Paste",this,SLOT(editPaste()));
    popgraph->insertSeparator();
    popgraph->insertItem("Proper&ties",this,SLOT(propGraph()));
 
@@ -173,9 +187,12 @@ GraphPanel::GraphPanel ( QWidget* parent, const char* name )
    popseries->insertSeparator();
    popseries->insertItem("Proper&ties",this,SLOT(propSeries()));
    popseries->insertItem("&Rename",this,SLOT(editRename()));
+   popseries->insertItem("&New",this,SLOT(editNew()));
    popseries->insertSeparator();
    popseries->insertItem("&Top",this,SLOT(propTop()));
    popseries->insertItem("&Bottom",this,SLOT(propBottom()));
+   popseries->insertItem("&Visible",this,SLOT(propVisible()),0,VISIBLE_ID);
+   popseries->setCheckable(TRUE);
    
    popxbaxis = new KPopupMenu("Bottom axis");
    CHECK_PTR(popxbaxis);
@@ -201,24 +218,24 @@ GraphPanel::~GraphPanel()
    if(spath!=NULL)delete spath;
 }
 
-void GraphPanel::graphClick(const QMouseEvent *m,int object,uint key)
+void GraphPanel::graphClick(int object,uint key, int bt, double x, double y)
 {
-   int bt=m->button();
-   if(bt==LeftButton && (object==OBJ_PLOT || object==OBJ_SERIES)){
-      float x,y;
-      x=graph->transform(m->pos().x(),xBottom);
-      y=graph->transform(m->pos().y(),yLeft);
-      
+   SamplinCurve *c;
+   int xoff=0,yoff=0;
+//   int bt=m->button();
+
+   if(/*bt==LeftButton */bt!=RightButton && (object==OBJ_PLOT || object==OBJ_SERIES)){
       emit pointShow(bt,x,y);      
    }
    if(bt==LeftButton || bt==RightButton)
      selectObject(object,key);
    if(bt==RightButton){
 //      graph->blinkObject(object,key);
-      QPoint pt=graph->mapToGlobal(QPoint(m->pos().x(),m->pos().y()));
+      QPoint pt=graph->mapToGlobal(QPoint(x/*m->pos().x()*/,y/*m->pos().y()*/));
       objectPopup(pt,object);
    }
    
+  
 }
 
 void GraphPanel::graphDoubleClick(const QMouseEvent *m,int object,uint key)
@@ -297,6 +314,7 @@ void GraphPanel::objectPopup(QPoint &pt,int obj)
     default:
       if((crv=currentCurve())!=0){
 	 popseries->setTitle(QString("Series [")+crv->name()+"]");
+	 popseries->setItemChecked(VISIBLE_ID,crv->visible());
 	 popseries->popup(pt);            	   
       }
       break;
@@ -410,6 +428,8 @@ void GraphPanel::replot(void)
 
 void GraphPanel::editCut()
 {
+   QBuffer *qb;
+   QDataStream *qs;
    int i;
    uint key;
    
@@ -420,7 +440,22 @@ void GraphPanel::editCut()
 //      printf("removing item %i key %i\n",i,key);
 
       if(clipboard_curve!=NULL)delete clipboard_curve;
-      clipboard_curve = new SamplinCurve( *(graph->findCurve(key) ) );
+      clipboard_curve = new SamplinCurve(/* *(graph->findCurve(key) ) */);
+
+      qb=new QBuffer();
+      qb->open(IO_ReadWrite);
+      qs=new QDataStream(qb);
+
+      *qs << *(graph->findCurve(key));
+      qb->at(0);
+      *qs >> *clipboard_curve;
+
+//      QStoredDrag *mydata = new QStoredDrag("application/x-SampLinCurve");
+//      mydata->setEncodedData(qb->buffer());
+//      QApplication::clipboard()->setData(mydata);
+      
+      delete qs;
+      delete qb;
       
       graph->removeCurve(key);
       graph->replot();
@@ -431,24 +466,75 @@ void GraphPanel::editCut()
 
 void GraphPanel::editCopy()
 {
+   QBuffer *qb;
+   QDataStream *qs;
+   
+   //KClipboard *cb=new KClipboard();
+//   QClipboard *cb=kapp->clipboard();
+   SamplinCurve *crv;
+  // QByteArray array;
+//   QDataStream str(array, IO_WriteOnly);
+   
    int i;
    uint key;
    
    i=list->currentItem();
    if(i>=SERIES_BEGIN && i!=-1){
       key=list->currentKey();
+
       if(clipboard_curve!=NULL)delete clipboard_curve;
-      clipboard_curve = new SamplinCurve( *(graph->findCurve(key) ) );
+      crv = new SamplinCurve(/* *(graph->findCurve(key) )*/ );
+      clipboard_curve=crv;
+
+      qb=new QBuffer();
+      qb->open(IO_ReadWrite);
+      qs=new QDataStream(qb);
+
+      *qs << *(graph->findCurve(key));
+      qb->at(0);
+      *qs >> *clipboard_curve;
+      
+      delete qs;
+      delete qb;
+      
+//      if(cb->open(IO_WriteOnly)){
+	 
+//	 debug("clip open");
+//	 str<<(*crv);
+//	 cb->setOctetStream(array);
+	 	 
+//	 cb->close();
+//      }
+      
+      
+      
+   
    }
+  // delete cb;
 }
 
 void GraphPanel::editPaste()
 {
+   QBuffer *qb;
+   QDataStream *qs;
+   
    SamplinCurve *crv;
    uint key;
    
    if(clipboard_curve!=NULL){
-      crv = new SamplinCurve(*clipboard_curve);
+      crv = new SamplinCurve(/**clipboard_curve*/);
+
+      qb=new QBuffer();
+      qb->open(IO_ReadWrite);
+      qs=new QDataStream(qb);
+
+      *qs << *clipboard_curve;
+      qb->at(0);
+      *qs >> *crv;
+      
+      delete qs;
+      delete qb;
+      
       key=graph->insertCurve(crv);
       graph->replot();
 
@@ -503,10 +589,10 @@ bool GraphPanel::export(const char *fn)
      return graph->exportPic(fname);
    else if(fname.find(EXPORT_BMP_EXT,fname.length()-strlen(EXPORT_BMP_EXT),TRUE)!=-1)   
      return graph->exportBMP(fname);
-   else return graph->exportData(fname);
+   else return graph->exportASCII(fname);
 }
 
-bool GraphPanel::exportData(const char *fn)
+bool GraphPanel::exportASCII(const char *fn)
 {
    QString fname;
    
@@ -514,10 +600,29 @@ bool GraphPanel::exportData(const char *fn)
    else fname=fn;
    
    if(!fname.isEmpty())
-     return graph->exportData(fname);
+     return graph->exportASCII(fname);
    else return 0;
 }
 
+bool GraphPanel::importASCII(const char *fn)
+{
+   int ret;
+   QString fname;
+   
+   if(fn==0)fname=selectFile("Import ASCII series",EXPORT_TXT_EXT);
+   else fname=fn;
+   
+   if(!fname.isEmpty()){
+      ret=graph->importASCII(fname);
+      graph->replot();
+      listFill();
+   }
+   else ret=0;
+   
+   return ret;
+}
+
+
 bool GraphPanel::exportPic(const char *fn)
 {
    QString fname;
@@ -527,7 +632,9 @@ bool GraphPanel::exportPic(const char *fn)
    
    if(!fname.isEmpty())
      return graph->exportPic(fname);
+   
    else return 0;
+
 }
 
 bool GraphPanel::exportBMP(const char *fn)
@@ -559,7 +666,7 @@ bool GraphPanel::save()
    if(strlen(graph->fileName())){
       ret=graph->save(NULL);
       if(!ret)
-	QMessageBox::warning(this, "Graph window", graph->fileError());     
+	QMessageBox::warning(this, "SampLinGraph", graph->fileError());     
    }
    else{
       ret=saveDlg(NULL);
@@ -578,11 +685,17 @@ bool GraphPanel::saveTempl()
    return saveTempl(NULL);
 }
 
-bool GraphPanel::exportData()
+bool GraphPanel::exportASCII()
 {
-   return exportData(NULL);
+   return exportASCII(NULL);
 }
 
+bool GraphPanel::importASCII()
+{
+   return importASCII(NULL);
+}
+
+
 bool GraphPanel::exportPic()
 {
    return exportPic(NULL);
@@ -625,14 +738,18 @@ void GraphPanel::propSeries()
    SamplinCurve *crv;
 
    crv=currentCurve();
-   if(grseries==NULL && crv!=NULL)
+   if(grcurve==NULL && crv!=NULL)
      {
-	grseries = new Graph_series(NULL,"Series options");
-	grseries->setData(crv);
-	connect(grseries, SIGNAL(update()), this, SLOT(replot()));
-	grseries->show();
-	delete grseries;
-	grseries=NULL;   
+	grcurve = new GraphSeries(NULL,"Series options");
+	grcurve->setData(crv);
+	connect(grcurve, SIGNAL(update()), this, SLOT(replot()));
+	
+	//	grseries = new Graph_series(NULL,"Series options");
+	//	grseries->setData(crv);
+	//	connect(grseries, SIGNAL(update()), this, SLOT(replot()));
+	grcurve->show();
+	delete grcurve;
+	grcurve=NULL;   
 //	listFill();
      }
 }
@@ -659,6 +776,30 @@ void GraphPanel::editRename()
    }
 }
 
+void GraphPanel::editNew()
+{
+   SamplinCurve *crv;
+   SamplinInput *inp;
+   int i;
+   
+   inp = new SamplinInput(this, "input");
+   inp->setTitle("Create new series");
+   inp->setText("");
+   inp->show();
+   if(inp->result()==QDialog::Accepted){
+      crv = new SamplinCurve();
+      crv->setName(inp->getText());
+      graph->insertCurve(crv);
+      listFill(FALSE);
+      list->setCurrentItem(list->count()-1);
+      list->setAutoUpdate(TRUE);
+      list->repaint();
+      graph->replot();
+   }
+   delete inp;
+}
+
+
 SamplinCurve *GraphPanel::currentCurve()
 {
    SamplinCurve *crv=NULL;
@@ -683,6 +824,15 @@ void GraphPanel::propBottom()
    if(list->currentItem()!=(int)list->count()-1)list->setCurrentItem(list->count()-1);
 }
 
+void GraphPanel::propVisible()
+{
+   SamplinCurve *crv=currentCurve();
+   
+   crv->setVisible(!(crv->visible()));
+   graph->replot();
+   listFill();
+}
+
 void GraphPanel::listHighlight(int index)
 {
    uint key=0;
@@ -713,21 +863,25 @@ bool GraphPanel::loadDlg(const char *n)
 {
    bool ret=FALSE;
    QString fn;
-   QFileDialog *fdlg;
-   QStrList *sl=new QStrList();
+   KFileDialog *fdlg;
+   QString filter;
 
-   sl->insert(0,"All files (*)");
-   sl->insert(0,"Old SampLin formats (*"OLD_GRAPH_EXT")");
-   sl->insert(0,"SampLin 1.3 Graph (*"GRAPH_EXT")");
+   if(isSaved()==FALSE)
+     if( QMessageBox::warning( this, "SampLinGraph",
+			      "Session contains unsaved data\n"
+			      "Discard old data ?",
+			      "&Ok", "&Cancel",
+			      0, 1 ) == 1 ) return FALSE;
+   
+   filter="*"GRAPH_EXT"|SampLin 1.3 graph (*"GRAPH_EXT")\n";
+   filter+="*"OLD_GRAPH_EXT"|Old SampLin graph (*"OLD_GRAPH_EXT")\n";
+   filter+="*|All files (*)";
 
    
    if(n==0){
-//      fdlg = new QFileDialog( cnf.root_dir+cnf.data_dir, 0/*"*"+QString(GRAPH_EXT)*/, this, "Open graph",TRUE );
-      fdlg = new QFileDialog( this, "Open graph",TRUE );
+      fdlg = new KFileDialog( root_dir, filter, this, "Open graph",TRUE, FALSE );
       fdlg->setCaption("Open graph");
-      fdlg->setFilters(*sl);
-      fdlg->setDir(cnf.root_dir+cnf.data_dir);
-
+      
       fdlg->show();
       fn = fdlg->selectedFile();
       if(fdlg->result()!=QDialog::Accepted)fn="";
@@ -735,37 +889,32 @@ bool GraphPanel::loadDlg(const char *n)
       //fn = QFileDialog::getOpenFileName( cnf.root_dir+cnf.data_dir, "*"+QString(SAMPLIN_EXT), this, "Open graph",TRUE );
    }
    else
-     fn= cnf.root_dir+cnf.data_dir + n;
+     fn= /*cnf.root_dir+cnf.data_dir +*/ n; //!!! set cwd  !!!
    
    if ( !fn.isEmpty() ) {
       if(fn.find(GRAPH_EXT,fn.length()-strlen(GRAPH_EXT),TRUE)==-1 &&
 	 fn.find(OLD_GRAPH_EXT,fn.length()-strlen(OLD_GRAPH_EXT),TRUE)==-1
 	 )fn+=GRAPH_EXT;
 
-	if(isSaved()==FALSE)
-	  if( QMessageBox::warning( this, "Graph window",
-				     "Session contains unsaved data\n"
-				     "Discard old data ?",
-				     "&Ok", "&Cancel",
-				     0, 1 ) == 1 ) return FALSE;
+
       
       ret=graph->load(fn);
       if(!ret)
-	QMessageBox::warning(this, "Graph window", graph->fileError());
+	QMessageBox::warning(this, "SampLinGraph", graph->fileError());
    }
 
-   delete sl;
    return ret;
 }
 
 bool GraphPanel::saveDlg(const char *n)
 {
+   
    QString fn;
    bool ret=FALSE;
-   QFileDialog *fdlg;
+   KFileDialog *fdlg;
    
    if(n==0){
-      fdlg = new QFileDialog( cnf.root_dir+cnf.data_dir, "*"+QString(GRAPH_EXT), this, "Save graph",TRUE );
+      fdlg = new KFileDialog( root_dir, "*"+QString(GRAPH_EXT), this, "Save graph",TRUE, FALSE );
       fdlg ->setCaption("Save graph");
       fdlg ->show();
       fn = fdlg->selectedFile();
@@ -774,14 +923,14 @@ bool GraphPanel::saveDlg(const char *n)
 //      fn = QFileDialog::getSaveFileName( cnf.root_dir+cnf.data_dir, "*"+QString(SAMPLIN_EXT), this, "Save graph" );
    }
    else
-     fn= cnf.root_dir+cnf.data_dir + n;
+     fn= /*cnf.root_dir+cnf.data_dir +*/ n; //!!! set cwd !!!
 
    if ( !fn.isEmpty() ) {
       if(fn.find(GRAPH_EXT,fn.length()-strlen(GRAPH_EXT),TRUE)==-1)fn+=GRAPH_EXT;
 
       QFile f(fn);
       if(f.exists()==TRUE)
-	switch( QMessageBox::warning( this, "Graph window",
+	switch( QMessageBox::warning( this, "SampLinGraph",
 					 "File already exists\n"
 					 "Do you want to overwrite old file ?",
 					 "&Ok", "&Cancel",
@@ -789,7 +938,7 @@ bool GraphPanel::saveDlg(const char *n)
 	 case 0: 
 	   ret=graph->save(fn);
 	   if(!ret)
-	     QMessageBox::warning(this, "Graph window", graph->fileError());
+	     QMessageBox::warning(this, "SampLinGraph", graph->fileError());
 	   break;
 	 case 1:
 	   break;
@@ -797,7 +946,7 @@ bool GraphPanel::saveDlg(const char *n)
       else {
 	 ret=graph->save(fn);
 	 if(!ret)
-	   QMessageBox::warning(this, "Graph window", graph->fileError());
+	   QMessageBox::warning(this, "SampLinGraph", graph->fileError());
       }
    }
    return ret;
@@ -806,9 +955,9 @@ bool GraphPanel::saveDlg(const char *n)
 QString GraphPanel::selectFile(const char *caption, const char *ext)
 {
    QString fn;
-   QFileDialog *fdlg;
+   KFileDialog *fdlg;
 
-   fdlg = new QFileDialog( cnf.root_dir+cnf.data_dir, "*"+QString(ext), this, caption,TRUE );
+   fdlg = new KFileDialog( root_dir, "*"+QString(ext), this, caption,TRUE, FALSE );
    fdlg ->setCaption(caption);
    fdlg ->show();
    fn = fdlg->selectedFile();
@@ -818,19 +967,24 @@ QString GraphPanel::selectFile(const char *caption, const char *ext)
    if ( !fn.isEmpty()) {
       if(!QFile::exists(fn) && fn.find(ext,fn.length()-strlen(ext),TRUE)==-1)fn+=ext;
 
-      if(QFile::exists(fn)==TRUE)
-	switch( QMessageBox::warning( this, "Graph window",
-					 "File already exists\n"
-					 "Do you want to overwrite old file ?",
-					 "&Ok", "&Cancel",
-					 0, 1 ) ) {
-	 case 0: 
-	   break;
-	 case 1:
-	   fn="";
-	   break;
-	}
+//      if(QFile::exists(fn)==TRUE)
+//	switch( QMessageBox::warning( this, "SampLinGraph",
+//					 "File already exists\n"
+//					 "Do you want to overwrite old file ?",
+//					 "&Ok", "&Cancel",
+//					 0, 1 ) ) {
+//	 case 0: 
+//	   break;
+//	 case 1:
+//	   fn="";
+//	   break;
+//	}
    }
    return fn;    
 }
 
+void GraphPanel::filenameChangedSlot()
+{
+   emit filenameChanged();
+   
+}
