@@ -29,7 +29,7 @@
 
 #include <stdio.h>
 
-static char *cstyleNames[]={"None","Line","Sticks","Steps","Dots","Spline",NULL
+static char *cstyleNames[]={"None","Line","Sticks","Steps","Dots","Spline","Fit",NULL
 };
 static char *lstyleNames[]={"None","Solid","Dash","Dot","Dash-dot","Dash-dot-dot",NULL
 };
@@ -38,6 +38,12 @@ static char *sstyleNames[]={"None","Circle","Rectangle","Diamond","Triangle",
 };
 
 
+#define isInsideR(x,y) ( rx1 <= x && rx2 >= x && ry1 <= y && ry2 >= y )
+#define isInside(x1,x2,x) ( qwtMin(x1,x2) <= x && qwtMax(x1,x2) >=x )
+
+//#define isInsideR(x,y) ( rx1 < x && rx2 > x && ry1 < y && ry2 > y )
+//#define isInside(x1,x2,x) ( qwtMin(x1,x2) < x && qwtMax(x1,x2) >x )
+
 int SamplinCurveData::pointDist(QPoint &pt, const QwtDiMap &mx, const QwtDiMap& my)
 {
    int dist=INT_MAX,new_dist;
@@ -49,14 +55,22 @@ int SamplinCurveData::pointDist(QPoint &pt, const QwtDiMap &mx, const QwtDiMap&
    y=pt.y();
    
    //   printf("mx:%i,my:%i\n",mx.contains(x), my.contains(y));
+
+
+   
+   setMap(mx,my);
    
    //   if(mx.contains(x) && my.contains(y))
    for(i=0;i<len;++i){
-      u = mx.transform(d_x[i]);
-      v = my.transform(d_y[i]);
+      u = d_xMap.transform(d_x[i]);
+      v = d_yMap.transform(d_y[i]);
       new_dist=(x-u)*(x-u)+(y-v)*(y-v);
       if(new_dist<dist)dist=new_dist;
    }
+
+//   if(d_xoff)mx.setOffset(0);
+//   if(d_yoff)mx.setOffset(0);
+   
    return dist;
 }
 
@@ -68,10 +82,17 @@ uint SamplinCurveData::size() const
 
 }
 
+bool SamplinCurveData::isEmpty() const
+{
+
+   return !(qwtMin(d_x.size(),d_y.size()) || (d_style==Fit && d_parser.isParsed()));
+
+}
+
 
 void SamplinCurveData::init(const char *name)
 {
-
+   d_visible=TRUE;
    d_xAxis=xBottom;
    d_yAxis=yLeft;
    
@@ -92,7 +113,15 @@ void SamplinCurveData::init(const char *name)
    d_dataStyle = XandY;
    d_begin=0;
    d_step=1;
-
+   d_xoff=0;
+   d_yoff=0;
+   
+//   d_ix=-1;d_fman=0;
+//   d_range1=0;d_range2=0;
+   d_parser.setData(&d_x,&d_y);
+   //updateFunctionRange();
+   
+   
    bcolor=white;
    updatePixmap();
 }
@@ -113,23 +142,6 @@ SamplinCurveData::~SamplinCurveData()
 
 }
 
-//------------------------------------------------------------
-//
-//.S	Name: SamplinCurveData::setStyle
-//	Set the curve's style
-//
-//.s	Parameters:
-//.p	CurveStyle cs
-//
-//.s	Return Type
-//	void
-//
-//.s	Description
-//	Valid styles are SamplinCurveData::NoCurve, SamplinCurveData::Lines, 
-//	SamplinCurveData::Sticks, SamplinCurveData::Steps, SamplinCurveDatas::Dots, and
-//	SamplinCurveData::Splines.
-//
-//------------------------------------------------------------
 void SamplinCurveData::setStyle(CurveStyle cs)
 {
 	d_style = cs;
@@ -192,6 +204,60 @@ void SamplinCurveData::setData(double *x, double *y, int size)
     d_yraw = false;
 
    d_dataStyle = XandY;
+   
+   update();
+}
+
+void SamplinCurveData::putPoint(int index, double x, double y)
+{
+   int i, xsize, ysize;
+   
+   xsize=d_x.size();
+   ysize=d_y.size();
+   index=qwtMin(index,xsize);
+   index=qwtMin(index,ysize);
+   if(index==-1)index=xsize;
+   index=qwtMax(index,0);
+   
+   
+   if(d_xraw==false){
+      d_x.resize(xsize+1);
+      for(i=index;i<xsize;++i)
+	d_x[i+1]=d_x[i];
+      d_x[xsize]=x;
+   }
+   if(d_yraw==false){
+      d_y.resize(ysize+1);
+      for(i=index;i<ysize;++i)
+	d_y[i+1]=d_y[i];
+      d_y[ysize]=y;
+   }
+   
+   
+   update();
+}
+
+void SamplinCurveData::removePoint(int index)
+{   int i, xsize, ysize;
+   
+   xsize=d_x.size();
+   ysize=d_y.size();
+   
+   if(index<0 || index > xsize-1 || index > ysize-1)return;
+   
+   if(index<xsize && d_xraw==false){
+      for(i=index;i<xsize-1;++i)
+	d_x[i]=d_x[i+1];
+      d_x.resize(xsize-1);
+   }
+
+   if(index<ysize && d_yraw==false){
+      for(i=index;i<ysize-1;++i)
+	d_y[i]=d_y[i+1];
+      d_y.resize(ysize-1);
+   }
+     
+   update();
 }
 
 void SamplinCurveData::setData(double beg, double stp, double *y, int size)
@@ -213,6 +279,8 @@ void SamplinCurveData::setData(double beg, double stp, double *y, int size)
    d_begin=beg;
    
    d_dataStyle = Yonly;
+   
+   update();
 }
 
 void SamplinCurveData::setData(double *x, double beg, double stp, int size)
@@ -235,6 +303,8 @@ void SamplinCurveData::setData(double *x, double beg, double stp, int size)
    d_begin=beg;
    
    d_dataStyle = Xonly;
+   
+   update();
 }
 
 //------------------------------------------------------------
@@ -265,6 +335,8 @@ void SamplinCurveData::setRawData(double *x, double *y, int size)
    d_yraw = true;
 
    d_dataStyle = XandY;
+   
+   update();
 }
 
 void SamplinCurveData::setRawData(double beg, double stp, double *y, int size)
@@ -287,6 +359,8 @@ void SamplinCurveData::setRawData(double beg, double stp, double *y, int size)
    d_begin=beg;
    
    d_dataStyle = Yonly;
+   
+   update();
 }
 
 void SamplinCurveData::setRawData(double *x, double beg, double stp, int size)
@@ -310,6 +384,8 @@ void SamplinCurveData::setRawData(double *x, double beg, double stp, int size)
    
    
    d_dataStyle = Xonly;
+   
+   update();
 }
 
 //------------------------------------------------------------
@@ -337,7 +413,16 @@ void SamplinCurveData::setName(const char *name)
 //------------------------------------------------------------
 double SamplinCurveData::minXValue() const
 {
-	return qwtGetMin(d_x.data(),d_x.size());
+   double ret;
+   
+   if(d_style==Fit && d_parser.rangeMan())
+     ret=qwtMin( qwtGetMin(d_x.data(),d_x.size()), d_parser.xMin());
+   else
+     ret= qwtGetMin(d_x.data(),d_x.size());
+   
+   
+//   debug("xmin=%lf",ret);
+   return ret;
 }
 
 //------------------------------------------------------------
@@ -351,7 +436,10 @@ double SamplinCurveData::minXValue() const
 //------------------------------------------------------------
 double SamplinCurveData::minYValue() const
 {
-	return qwtGetMin(d_y.data(), d_y.size());
+   if(d_style==Fit && d_parser.rangeMan())
+     return qwtMin( qwtGetMin(d_y.data(), d_y.size()), d_parser.yMin()) ;
+   else
+     return qwtGetMin(d_y.data(), d_y.size());
 }
 
 //------------------------------------------------------------
@@ -365,7 +453,16 @@ double SamplinCurveData::minYValue() const
 //------------------------------------------------------------
 double SamplinCurveData::maxXValue() const
 {
-	return qwtGetMax(d_x.data(), d_x.size());
+   double ret;
+   
+   if(d_style==Fit && d_parser.rangeMan())
+     ret= qwtMax( qwtGetMax(d_x.data(), d_x.size()), d_parser.xMax());
+   else
+     ret= qwtGetMax(d_x.data(), d_x.size());
+   
+   //debug("xMax=%lf",ret);
+   
+   return ret;
 }
 
 //------------------------------------------------------------
@@ -379,7 +476,10 @@ double SamplinCurveData::maxXValue() const
 //------------------------------------------------------------
 double SamplinCurveData::maxYValue() const
 {
-	return qwtGetMax(d_y.data(), d_y.size());
+   if(d_style==Fit && d_parser.rangeMan())
+     return qwtMax( qwtGetMax(d_y.data(), d_y.size()), d_parser.yMax());
+   else
+     return qwtGetMax(d_y.data(), d_y.size());
 }
 
 
@@ -475,28 +575,48 @@ void SamplinCurve::draw(QPainter *p, const QRect &r)
 void SamplinCurve::draw(QPainter *p)
 {
 //   p->setRasterOp(OrROP);
+
+   rx1 = qwtMin(d_xMap.c1(),d_xMap.c2());
+   rx2 = qwtMax(d_xMap.c1(),d_xMap.c2());
+   ry1 = qwtMin(d_yMap.c1(),d_yMap.c2());
+   ry2 = qwtMax(d_yMap.c1(),d_yMap.c2());
    
-   if(d_x.size()==0 || d_y.size()==0)return;
+   p->setBrush(NoBrush);
+/*   p->drawLine(d_xMap.c1(),d_yMap.c1(),d_xMap.c1(),d_yMap.c2()); 
+   p->drawLine(d_xMap.c2(),d_yMap.c1(),d_xMap.c2(),d_yMap.c2()); 
+   p->drawLine(d_xMap.c1(),d_yMap.c1(),d_xMap.c2(),d_yMap.c1()); 
+   p->drawLine(d_xMap.c1(),d_yMap.c2(),d_xMap.c2(),d_yMap.c2()); */
+
+//   debug("double margins: %f,%f,%f,%f",rx1,rx2,ry1,ry2);
+//   debug("int margins: %i,%i,%i,%i",d_xMap.i1(),d_xMap.i2(),d_yMap.i1(),d_yMap.i2());
+   
+//   debug("curve style %i",d_style);
+   
+   if(isEmpty())return;
+
    switch (d_style)
-    {
-    case NoCurve:
+     {
+      case NoCurve:
 	break;
-    case Lines:
+      case Lines:
 	drawLines(*p);
 	break;
-    case Sticks:
+      case Sticks:
 	drawSticks(*p);
 	break;
-    case Steps:
+      case Steps:
 	drawSteps(*p);
 	break;
-    case Dots:
+      case Dots:
 	drawDots(*p);
 	break;
-    case Spline:
+      case Spline:
 	drawSpline(*p);
 	break;
-    default:
+      case Fit:
+	drawFit(*p);
+	break;
+     default:
 	drawDots(*p);
     }
     if (d_sym.style() != QwtSymbol::None)
@@ -526,42 +646,18 @@ void SamplinCurve::draw(QPainter *p)
 void SamplinCurve::drawLines(QPainter &p)
 {
    int i;
-   int u1,v1,u2,v2;
-   int x0,y0;
-   QRect r=p.window();
-   x0=r.left();
-   y0=r.top();
+   double u1,v1,u2,v2;
    
    int size = qwtMin(d_x.size(), d_y.size());
 	
-	if (d_pa.size() != uint(size))
-	{
-		if(!d_pa.resize(uint(size)))
-		   return;
-	}
-	
-	p.setPen(d_pen);
-
-	transform(d_x[0], d_y[0], u1, v1);
-
-   d_pa.setPoint(0,u1,v1);
-   p.moveTo(u1,v1);
-   p.lineTo(u1,v1);
+   p.setPen(d_pen);
    
    for (i=1;i<size;i++)
 	{
 	   transform(d_x[i], d_y[i], u2,v2);
-	   d_pa.setPoint(i,u2,v2);
-
-//	   drawClipLine(p,r,x0+u1,y0+v1,x0+u2,y0+v2);
-//!!	   drawClipLine(p,u1,v1,u2,v2);	   
-
-//	   printf("X:%f,tX:%i,Y:%f,tY:%i\n",d_x[i-1],u1,d_y[i-1],v1);
-	   p.lineTo(u2,v2);
-		u1 = u2;
-		v1 = v2;
+	   transform(d_x[i-1],d_y[i-1],u1,v1);
+	   drawClipLine(p, u1, v1, u2, v2);
 	}
-//	p.drawPolyline(d_pa);
 }
 
 
@@ -586,19 +682,22 @@ void SamplinCurve::drawLines(QPainter &p)
 //------------------------------------------------------------
 void SamplinCurve::drawSpline(QPainter &p)
 {
+ 
+   double xmin, xmax, ymin, ymax, delta;
+   double u1,v1,u2,v2;
+   double *param;
+   double dtmp;
+   int i;
+   int stype;
+   int rc;
+   int size = qwtMin(d_x.size(), d_y.size());
+   double *txval = new double[size];
+   double *tyval = new double[size];
+	
 
-    double xmin, xmax, ymin, ymax, delta;
-    double *param;
-    double dtmp;
-    int i;
-    int stype;
-    int rc;
-    int size = qwtMin(d_x.size(), d_y.size());
-    double *txval = new double[size];
-    double *tyval = new double[size];
-	
-	
-    if ( (!txval) | (!tyval) | (!d_pa.resize(d_splineSize)) )
+   p.setPen(d_pen);
+   
+    if ( (!txval) | (!tyval)/* | (!d_pa.resize(d_splineSize))*/ )
     {
 	if (txval) delete[] txval;
 	if (tyval) delete[] tyval;
@@ -678,9 +777,15 @@ void SamplinCurve::drawSpline(QPainter &p)
 		delta = param[size - 1] / double(d_splineSize-1);
 		for (i=0;i<d_splineSize;i++)
 		{
-		    dtmp = delta * double(i);
-		    d_pa.setPoint(i, int(rint(d_spx.value(dtmp))),
-				  int(rint(d_spy.value(dtmp))));
+		   dtmp = delta * double(i);
+		   
+		   u1=u2;
+		   v1=v2;
+		   u2=int(rint(d_spx.value(dtmp)));
+		   v2=int(rint(d_spy.value(dtmp)));
+		   if(i>0)drawClipLine(p,u1,v1,u2,v2);
+		   //d_pa.setPoint(i, int(rint(d_spx.value(dtmp))),
+//				  int(rint(d_spy.value(dtmp))));
 		}
 	    }
 	    
@@ -711,9 +816,16 @@ void SamplinCurve::drawSpline(QPainter &p)
 	    
 	    for (i=0;i<d_splineSize;i++)
 	    {
-		dtmp = ymin + delta * double(i);
-		d_pa.setPoint(i, int(rint(d_spx.value(dtmp))),
-			      int(rint(dtmp)));
+	       dtmp = ymin + delta * double(i);
+
+	       u1=u2;
+	       v1=v2;
+	       u2=int(rint(d_spx.value(dtmp)));
+	       v2=int(rint(dtmp));
+	       if(i>0)drawClipLine(p,u1,v1,u2,v2);
+	       
+	       //		d_pa.setPoint(i, int(rint(d_spx.value(dtmp))),
+//			      int(rint(dtmp)));
 	    }
 	}
     }		
@@ -741,15 +853,22 @@ void SamplinCurve::drawSpline(QPainter &p)
 	    
 	    for (i=0;i<d_splineSize;i++)
 	    {
-		dtmp = xmin + delta * double(i);
-		d_pa.setPoint(i, int(rint(dtmp)),
-			      int(rint(d_spy.value(dtmp))));
+	       dtmp = xmin + delta * double(i);
+
+	       u1=u2;
+	       v1=v2;
+	       u2=int(rint(dtmp));
+	       v2=int(rint(d_spy.value(dtmp)));
+	       if(i>0)drawClipLine(p,u1,v1,u2,v2);
+	       
+//		d_pa.setPoint(i, int(rint(dtmp)),
+//			      int(rint(d_spy.value(dtmp))));
 	    }
 	}
     }
     
-    p.setPen(d_pen);
-    p.drawPolyline(d_pa);
+
+//    p.drawPolyline(d_pa);
     
     delete[] txval;
     delete[] tyval;
@@ -815,8 +934,7 @@ void SamplinCurveData::setSplineSize(int s)
 void SamplinCurve::drawSticks(QPainter &p)
 {
     int i;
-    int u,v;
-    int v0;
+    double u,v, v0;
     int size = qwtMin(d_x.size(), d_y.size());
     
     if (qwtMax(d_yMap.d1(), d_yMap.d2()) < 0.0)
@@ -824,15 +942,15 @@ void SamplinCurve::drawSticks(QPainter &p)
     else if (qwtMin(d_yMap.d1(), d_yMap.d2()) > 0.0)
        v0 = qwtMax(d_yMap.i1(), d_yMap.i2());
     else
-       v0 = d_yMap.transform(0.0);
+       v0 = d_yMap.xTransform(0.0);
     
     p.setPen(d_pen);
     
     for (i=0;i<size;i++)
-    {
+     {
 	transform(d_x[i], d_y[i], u,v);
-	p.drawLine(u,v0,u,v);
-    }
+	drawClipLine(p, u, v0, u, v);
+     }
     
 }
 
@@ -858,15 +976,16 @@ void SamplinCurve::drawSticks(QPainter &p)
 void SamplinCurve::drawDots(QPainter &p)
 {
     int i;
-    int u,v;
+    double u,v;
     int size = qwtMin(d_x.size(), d_y.size());
     
     p.setPen(d_pen);
     
     for (i=0;i<size;i++)
     {
-	transform(d_x[i], d_y[i], u,v);
-	p.drawPoint(u,v);
+       transform(d_x[i], d_y[i], u,v);
+       if(isInsideR(u,v))
+	 p.drawPoint(u,v);
     }
     
 }
@@ -894,32 +1013,38 @@ void SamplinCurve::drawDots(QPainter &p)
 void SamplinCurve::drawSteps(QPainter &p)
 {
     int i;
-    int u1,v1,u2,v2,w1,w2;
+    double u1,v1,u2,v2;
+    int w1,w2;
     int size = qwtMin(d_x.size(), d_y.size());
     
+   /*
     if (d_pa.size() != uint (2*size - 1))
     {
 	if (!d_pa.resize(uint(2*size - 1)))
 	   return;
-    }
+    }*/
     
     p.setPen(d_pen);
     
-    w1 = (d_pen.width() +1) /2;
-    w2 = d_pen.width() / 2;
+//    w1 = (d_pen.width() +1) /2;
+//    w2 = d_pen.width() / 2;
     
-    transform(d_x[0], d_y[0], u1, v1);
-    d_pa.setPoint(0, u1,v1);   
+//    transform(d_x[0], d_y[0], u1, v1);
+//    d_pa.setPoint(0, u1,v1);   
     
     for (i=1;i<size;i++)
     {
-	transform(d_x[i], d_y[i], u2,v2);
-	d_pa.setPoint(2*i - 1, u2,v1);   
-	d_pa.setPoint(2*i, u2,v2);
-	u1 = u2;
-	v1 = v2;
+       transform(d_x[i-1], d_y[i-1], u1,v1);
+       transform(d_x[i], d_y[i], u2,v2);
+       drawClipLine(p,u1,v1,u2,v1);
+       drawClipLine(p,u2,v1,u2,v2);
+       
+//       d_pa.setPoint(2*i - 1, u2,v1);   
+//	d_pa.setPoint(2*i, u2,v2);
+//	u1 = u2;
+//	v1 = v2;
     }
-    p.drawPolyline(d_pa);
+//    p.drawPolyline(d_pa);
     
     
 }
@@ -943,10 +1068,10 @@ void SamplinCurve::drawSteps(QPainter &p)
 //.f	void SamplinCurveData::transform(double x, double y, int &u, int &v)
 //
 //------------------------------------------------------------
-void SamplinCurveData::transform(double x, double y, int &u, int &v)
+void SamplinCurveData::transform(double x, double y, double &u, double &v)
 {
-    u = d_xMap.transform(x);
-    v = d_yMap.transform(y);
+    u = d_xMap.xTransform(x);
+    v = d_yMap.xTransform(y);
 }
 
 
@@ -971,8 +1096,8 @@ void SamplinCurveData::transform(double x, double y, int &u, int &v)
 //------------------------------------------------------------
 void SamplinCurve::drawSymbols(QPainter &p)
 {
-   QPoint pt;
-   int i, u, v;
+   int i;
+   double u, v;
    int size = qwtMin(d_x.size(), d_y.size());
     
    int w2 = d_sym.size().width() / 2;
@@ -981,8 +1106,7 @@ void SamplinCurve::drawSymbols(QPainter &p)
     for (i=0;i<size;i++)
      {
 	transform(d_x[i], d_y[i], u,v);
-	pt.setX(u);pt.setY(v);
-//	if(clipRect.contains(pt))
+	if(isInsideR(u,v))
 	  d_sym.draw(&p, u - w2, v - h2);
      }
 }
@@ -1060,131 +1184,25 @@ void SamplinCurveData::setMap(const QwtDiMap &xmap, const QwtDiMap &ymap)
 {
    d_xMap = xmap;
    d_yMap = ymap;
+   
+   d_xMap.setOffset(d_xoff);
+   d_yMap.setOffset(-d_yoff);
+
+   /*
    clipRect.setLeft(d_xMap.i1());
    clipRect.setRight(d_xMap.i2());
    clipRect.setTop(d_yMap.i2());
    clipRect.setBottom(d_yMap.i1());
+    */
 }
 
-void SamplinCurve::drawClipLine(QPainter &p,int x1, int y1, int x2, int y2)
-  {
-     QPoint p1(x1,y1), p2(x2,y2),l1,l2;
-     bool v1,v2,w1,w2,valid1,valid2;
-     
-
-     
-     //     printf("clip: %i,%i,%i,%i\n",r.left(),r.right(),r.top(),r.bottom());
-     
-     v1=clipRect.contains(p1);
-     v2=clipRect.contains(p2);
-
-     //     if(d_x.contains(x1) && d_y.contains(y1) && d_x.contains(x2) && d_y.contains(y2))
-     if( v1 && v2)
-       p.drawLine(x1,y1,x2,y2);
-     else {
-	w1=clipRect.contains(p1,TRUE);
-	w2=clipRect.contains(p2,TRUE);
-	if(w1){
-	   p.drawLine(p1,intersect(clipRect,p1,p2));
-	}
-	if(w2){
-	   p.drawLine(p2,intersect(clipRect,p2,p1));
-	}
-	if(!w1 && !w2){
-	   l1=intersect(clipRect,p1,p2,&valid1);
-	   l2=intersect(clipRect,p2,p1,&valid2);
-	   printf("v1=%i,v2=%i\n",valid1,valid2);
-	   if(valid1 && valid2){
-	      p.drawLine(l1,l2);
-	      printf("ok\n");
-  
-	   }
-	}
-     }
-     
-     
-  }
-
-QPoint SamplinCurve::intersect(QRect &r,QPoint &p1, QPoint &p2, bool *valid=0)
-  {
-     int x1,y1,x2,y2;
-     int rx1,ry1,rx2,ry2;
-     int t;
-     QPoint ret=p1;
-     bool flag=FALSE;
-     
-     x1=p1.x();x2=p2.x();y1=p1.y();y2=p2.y();
-     rx1=qwtMin(r.left(),r.right());
-     rx2=qwtMax(r.left(),r.right());
-     ry1=qwtMin(r.top(),r.bottom());
-     ry2=qwtMax(r.top(),r.bottom());     
-
-
-     if(x1==x2){
-	ret.setX(x1);
-	if(y2>y1)
-	  ret.setY(ry2);
-	else ret.setY(ry1);
-	flag=TRUE;
-     }
-     if(y1==y2){
-	ret.setY(y1);
-	if(x2>x1)
-	  ret.setX(rx2);
-	else ret.setX(rx1);
-	flag=TRUE;
-     }
-     
-     if(x1<x2){
-	//rx2
-	t=y1+(double)(y2-y1)/(x2-x1)*(double)(rx2-x1);
-	if(t<=ry2 && t>=ry1){
-	   ret.setX(rx2);
-	   ret.setY(t);
-	   flag=TRUE;
-	}
-     }
-     
-     if(x1>x2){
-	//rx1
-	t=y1+(double)(y2-y1)/(x2-x1)*(double)(rx1-x1);
-	if(t<=ry2 && t>=ry1){
-	   ret.setX(rx1);
-	   ret.setY(t);
-	   flag=TRUE;
-	}
-     }
-
-     if(y1<y2){
-	//ry2
-	t=x1+(double)(x2-x1)/(y2-y1)*(double)(ry2-y1);
-	if(t<=rx2 && t>=rx1){
-	   ret.setY(ry2);
-	   ret.setX(t);
-	   flag=TRUE;
+ 
+void SamplinCurveData::setVisible(bool vis){
+   d_visible=vis;
+   updatePixmap();
 }
-     }
-     
-     if(y1>y2){
-	//ry1
-	t=x1+(double)(x2-x1)/(y2-y1)*(double)(ry1-y1);
-	if(t<=rx2 && t>=rx1){
-	   ret.setY(ry1);
-	   ret.setX(t);
-//	   printf("clip %i,%i,%i,%i at y=%i; %i",x1,y1,x2,y2,ry1,t);
-	   flag=TRUE;
-	}
-     }
-
-     if(valid!=0){
-	if(ret.x()>=rx1 && ret.x()<=rx2 && ret.y()>=ry1 && ret.y()<=ry2 && flag)
-	  *valid=TRUE;
-	else *valid=FALSE;
-     }
-     
-     return(ret);
-  }
-
+ 
+ 
 
 void SamplinCurve::updatePixmap(void){
 
@@ -1199,13 +1217,18 @@ void SamplinCurve::updatePixmap(void){
    p.begin(&pix);
    p.eraseRect(0,0,16,16);
    p.fillRect(2,3,13,13,QBrush(bcolor));
-   p.setPen(d_pen);
-   if(d_style!=NoCurve)
-     p.drawLine(2,9,15,9);
-   d_sym.draw(&p,8-xoff,9-yoff);
-   p.setPen(black);
-   p.setBrush(NoBrush);
    p.end();
+   
+   if(d_visible){
+      p.begin(&pix);
+      p.setPen(d_pen);
+      if(d_style!=NoCurve)
+	p.drawLine(2,9,15,9);
+      d_sym.draw(&p,8-xoff,9-yoff);
+      p.setPen(black);
+      p.setBrush(NoBrush);
+      p.end();
+   }
 
    p.begin(&bit);
    p.eraseRect(0,0,16,16);
@@ -1220,6 +1243,28 @@ QPixmap &SamplinCurve::pixmap(void){
    return pix;
 }
 
+void SamplinCurve::drawIcon(QPainter &p, const QRect &r)
+{
+   int xoff, yoff;
+   
+   xoff=d_sym.size().width()/2;
+   yoff=d_sym.size().height()/2;
+   
+   p.fillRect(r.x(),r.y(),r.width(),r.height(),QBrush(bcolor));
+   
+   if(d_visible){
+      if(d_style!=NoCurve){
+	 p.setPen(d_pen);
+	 p.drawLine(r.x(),(r.top()+r.bottom())/2,r.right(),(r.top()+r.bottom())/2);
+//	 debug("icon line %i - %i",r.x(),r.right());
+      }
+      d_sym.draw(&p,(r.right()+r.left())/2-xoff,(r.top()+r.bottom())/2-yoff);
+      p.setPen(black);
+      p.setBrush(NoBrush);
+   }
+}
+ 
+ 
 void SamplinCurveData::setBackgroundColor(const QColor &c){
 
    if(bcolor!=c){
@@ -1235,10 +1280,14 @@ QDataStream &operator<<( QDataStream &s, SamplinCurveData &series ){
    len = series.size();
    
    s << series.name();
+//   debug("%i,%s",strlen(series.name().data()),series.name().data());
    s << series.legend();
+//   debug("%i,%s",strlen(series.legend().data()),series.legend().data());
    
    s << series.xAxis();
+//   debug("%i",series.xAxis());
    s << series.yAxis();
+//   debug("%i",series.yAxis());
    
    s << series.style();
    s << series.pen();
@@ -1252,7 +1301,9 @@ QDataStream &operator<<( QDataStream &s, SamplinCurveData &series ){
    s << series.splineSize();
    s << series.splineType();
      
-   s << (int)0; //reserved for extension
+   s << (int)(!series.visible()); //is visible ?
+   s << (int)1; //d_parser follows
+   s << series.d_parser; //save parser data
    s << (int)0; //reserved for extension
    
    s << len;
@@ -1276,8 +1327,8 @@ QDataStream &operator<<( QDataStream &s, SamplinCurveData &series ){
 	   else s << series.y(len);
       }
    }
-   
-     return(s);
+
+   return(s);
 }
 
 QDataStream &operator>>( QDataStream &s, SamplinCurveData &series ){
@@ -1301,6 +1352,13 @@ QDataStream &operator>>( QDataStream &s, SamplinCurveData &series ){
    s >> i;
    series.setyAxis(i);
    
+//   debug("%i,%s",strlen(series.name().data()),series.name().data());
+//   debug("%i,%s",strlen(series.legend().data()),series.legend().data());
+   
+//   debug("%i",series.xAxis());
+//   debug("%i",series.yAxis());
+   
+   
    s >> i;
    series.setStyle((SamplinCurveData::CurveStyle)i);
    s >> p;
@@ -1317,7 +1375,13 @@ QDataStream &operator>>( QDataStream &s, SamplinCurveData &series ){
    series.setSplineType(i);
    
    s >> i; //reserved for extension
-   s >> i; //reserved for extension
+   series.setVisible(!i);
+
+   s >> i; //was parser saved (ver>=1.5.0)
+   if(i==1){ 
+      s >> series.d_parser;
+      s >> i; //reserved for extension
+   }
    
    s >> count;
    len=count;
@@ -1396,3 +1460,222 @@ int SamplinCurveData::symbolStyle(const char *str)
      return -1;
   }
 
+void SamplinCurveData::setOffset(int x, int y)
+{
+   d_xoff=x;
+   d_yoff=y;
+}
+
+int SamplinCurveData::xOffset(void)
+{
+   return d_xoff;
+}
+
+int SamplinCurveData::yOffset(void)
+{
+   return d_yoff;
+}
+
+
+void SamplinCurve::drawClipLine(QPainter &p, double x1, double y1, double x2, double y2)
+{
+//   double x1, x2, y1, y2;
+   double dx,dy;
+   double cx[2], cy[2], tmp;
+   int done=0;
+   int u1, v1, u2, v2;
+
+//   x1=d_xMap.xTransform(_x1);
+//   x2=d_xMap.xTransform(_x2);
+//   y1=d_yMap.xTransform(_y1);
+//   y2=d_yMap.xTransform(_y2);
+
+//   if(!finite(x1) || !finite(x2) || !finite(y1) || !finite(y2))return;
+   
+   dx=x2-x1;
+   dy=y2-y1;
+   
+//   debug("drawline %lf,%f,%f,%f",x1,y1,x2,y2);
+   
+   if(isInside(rx1,rx2,x1) && isInside(ry1,ry2,y1)){
+      cx[done]=x1;
+      cy[done]=y1;
+      ++done;
+//      printf("*L1");
+   }
+   
+   if(isInside(rx1,rx2,x2) && isInside(ry1,ry2,y2)){
+      cx[done]=x2;
+      cy[done]=y2;
+      ++done;
+//      printf("*L2");
+   }
+
+   if(done==2){
+      p.drawLine(cx[0],cy[0],cx[1],cy[1]);
+      return;
+   }
+   
+   if(x1<rx1 && x2<rx1)return;
+   if(x1>rx2 && x2>rx2)return;
+   if(y1<ry1 && y2<ry1)return;
+   if(y1>ry2 && y2>ry2)return;
+   
+   //test top edge
+   if(done<2)
+     if(dy!=0 && isInside(y1,y2,ry1)){ 
+	tmp = dx/dy*(ry1-y1)+x1;
+	if(finite(tmp) && isInside(rx1,rx2,tmp)){
+	   cx[done]=tmp;
+	   cy[done]=ry1;
+	   ++done;
+//	   printf("*L3");
+	}
+     }
+   //test bottom edge
+   
+   if(done<2)
+     if(dy!=0 && isInside(y1,y2,ry2)){ 
+	tmp = dx/dy*(ry2-y1)+x1;
+	if(finite(tmp) && isInside(rx1,rx2,tmp)){
+	   cx[done]=tmp;
+	   cy[done]=ry2;
+	   ++done;
+//	   printf("*L4");
+	}
+     }
+   
+      //test left edge
+   if(done<2)
+     if(dx!=0 && isInside(x1,x2,rx1)){ 
+	tmp = dy/dx*(rx1-x1)+y1;
+	if(finite(tmp) && isInside(ry1,ry2,tmp)){
+	   cx[done]=rx1;
+	   cy[done]=tmp;
+	   ++done;
+//	   printf("*L5");
+	}
+     }
+   
+   //test right edge
+   if(done<2)
+     if(dx!=0 && isInside(x1,x2,rx2)){ 
+	tmp = dy/dx*(rx2-x1)+y1;
+	if(finite(tmp) && isInside(ry1,ry2,tmp)){
+	   cx[done]=rx2;
+	   cy[done]=tmp;
+	   ++done;
+//	   printf("*L6");
+	}
+     }
+   
+   if(done==2){
+      p.drawLine(cx[0],cy[0],cx[1],cy[1]);
+//    printf("\ndrawline %f,%f,%f,%f -> %f,%f,%f,%f\n",
+//	     x1,y1,x2,y2,cx[0],cy[0],cx[1],cy[1]);
+   }
+//   else printf("sorry\n");
+}
+
+/*
+int SamplinCurveData::setFunction(const char *s)
+{
+   return d_parser.setFunction(s);
+}
+
+
+void SamplinCurveData::updateFunctionRange(void)
+{
+   double step,var,tmp;
+   int i;
+   
+   d_fmin=DBL_MAX;d_fmax=DBL_MIN;
+   d_vmin=DBL_MAX;d_vmax=DBL_MIN;   
+
+   if(d_fman && d_range1!=d_range2 && d_parser.isParsed()){
+      d_vmin=qwtMin(d_range1,d_range2);
+      d_vmax=qwtMax(d_range1,d_range2);
+      
+      step=(d_vmax-d_vmin)/2048;
+      var=d_vmin;
+      for(i=0;i<=2048;++i){
+	 tmp=d_parser.function(var);
+//	 if(isinf(tmp)==-1)tmp=-DBL_MAX/10;
+//	 if(isinf(tmp)==1)tmp=DBL_MAX/10;
+	 if(!isnan(tmp) && !isinf(tmp)){
+	    if(tmp>d_fmax)d_fmax=tmp;
+	    if(tmp<d_fmin)d_fmin=tmp;
+	 }
+	 var+=step;
+      }
+//   debug("vmin=%f, vmax=%f, fmin=%f, fmax=%f",d_vmin,d_vmax,d_fmin,d_fmax);
+   }
+
+
+   
+}
+*/
+
+void SamplinCurve::drawFit(QPainter &p)
+{
+   int i;
+   double u1,v1,u2,v2;
+
+   double vrange,delta,var,res,oldvar,oldres;
+   double b1,b2;
+   int npoints;
+   bool isfirst=TRUE;
+   
+   //if(d_ix==-1)return;
+   if(!d_parser.isParsed())return;
+   
+
+   if(d_parser.rangeMan()){
+      b1=d_parser.rangeLow();
+      b2=d_parser.rangeHigh();
+   }
+   else if(d_x.size() && d_y.size()){
+      b1=minXValue();
+      b2=maxXValue();
+   }
+   else {
+      b1=d_xMap.invTransform(d_xMap.c1());
+      b2=d_xMap.invTransform(d_xMap.c2());
+//      debug("draw: empty, b1=%lf,b2=%lf, c1=%i,c2=%i, d1=%lf,d2=%lf",b1,b2,d_xMap.c1(),d_xMap.c2(),d_xMap.d1(),d_xMap.d2());
+   }
+
+   vrange=fabs(b2-b1);
+   npoints=2*(int)fabs(d_xMap.transform(b1)-d_xMap.transform(b2));
+   delta=(b2-b1)/(float)npoints;
+
+   p.setPen(d_pen);
+   
+   var=b1;
+//   oldvar=b1;
+//   oldres=d_parser.deffkt(b1);
+   
+   for(i=0;i<npoints+1;++i){
+      res=d_parser.function(var);
+      if(!isnan(res) && !isinf(res)){
+	 transform(var, res, u2,v2);
+	 if(!isfirst){
+	    transform(oldvar,oldres,u1,v1);
+	    drawClipLine(p, u1, v1, u2, v2);
+	 }
+	 isfirst=FALSE;
+      }else isfirst=TRUE;
+
+      oldvar=var;
+      oldres=res;
+      var+=delta;
+   }
+
+}
+
+void SamplinCurve::update(void)
+{
+   if(d_style==Fit && d_parser.isParsed() && d_parser.autoFit()){
+      d_parser.regress();
+   }
+   
+}
