@@ -23,6 +23,7 @@
 #include <limits.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <regex.h>
 
 #include <qwt_math.h>
 
@@ -84,9 +85,18 @@ void SamplinPlotData::init(void)
    d_cColor=QColor(black);
    d_lFont=QFont("Helvetica",8);
    d_cFont=QFont("Helvetica",8);
+
+   d_stack=FALSE;
+   d_stackx=0;
+   d_stacky=0;
    
 }
 
+void SamplinPlotData::setFileName(const char *str)
+{
+   filename=str;
+}
+
 int SamplinPlotData::axis(const char *str)
 {
    char **s=axisNames;
@@ -571,8 +581,11 @@ void SamplinPlotData::receiveData(QDataStream &s){
    char b;
    
    removeCurves();
+
+   d_stack=FALSE;
+   
    s >> str;
-   setFileName(str);
+//   setFileName(str);
 
    s >> d_title;
    s >> d_titleFont;
@@ -594,7 +607,23 @@ void SamplinPlotData::receiveData(QDataStream &s){
       s >> d_printfit;
       s >> d_printwidth;
       s >> d_printheight;
+   }else if(i==10){
+      s >> d_printfit;
+      s >> d_printwidth;
+      s >> d_printheight;
+      s >> d_stack;
+      s >> d_stackx;
+      s >> d_stacky;
+   }else if(i==9){
+      s >> d_printfit;
+      s >> d_printwidth;
+      s >> d_printheight;
+      s >> d_printinfo;
+      s >> d_stack;
+      s >> d_stackx;
+      s >> d_stacky;
    }
+   
    s >> i;
    if(i==12){
       s >> d_lpos;
@@ -660,10 +689,14 @@ void SamplinPlotData::sendData(QDataStream &s){
       s << *d_scale[i];
    }
    
-   s << (int)11;
+   s << (int)9;
    s << d_printfit;
    s << d_printwidth;
    s << d_printheight;
+   s << d_printinfo;
+   s << d_stack;
+   s << d_stackx;
+   s << d_stacky;
    s << (int)12;
    s << d_lpos;
    s << d_lColor;
@@ -709,6 +742,8 @@ bool SamplinPlotData::load(const char *fn)
    tmp=filename;
    if(fn!=NULL)tmp=fn;
    
+   setBusy(TRUE);
+   
    QFile f(tmp);
    if((ret=f.open( IO_ReadOnly))==TRUE){
       QDataStream s(&f);
@@ -754,9 +789,16 @@ bool SamplinPlotData::load(const char *fn)
       
       f.close();
       autoRefresh();
+
+      setFileName(f.name());
       setSaved();
+
+      //!!!!!!!!!! add setFileName and remove from >>, oldload, oldload2
    }
    else d_fileerror="Can't open file "+tmp+" for reading";
+
+   setBusy(FALSE);
+   
    return(ret);
    
 }
@@ -772,9 +814,11 @@ void SamplinPlotData::oldload(QDataStream &s){
    QArray<double> data;
    
    removeCurves();
+
+   d_stack=FALSE;
    
    s >> str;
-   setFileName(str);
+   //setFileName(str);
    s >> str;
    setLabel(str);
    s >> str;
@@ -842,8 +886,11 @@ void SamplinPlotData::oldload2(QDataStream &s){
    QPen p;
    
    removeCurves();
+
+   d_stack=FALSE;
+   
    s >> str;
-   setFileName(str);
+   //setFileName(str);
 
    s >> d_title;
    s >> d_titleFont;
@@ -908,6 +955,8 @@ bool SamplinPlotData::save(const char *fn)
    d_fileerror="";
    tmp=filename;
    if(fn!=NULL)tmp=fn;
+
+   setBusy(TRUE);
    
    QFile f(tmp);
    if(f.exists()==TRUE)rename(tmp,tmp+"~");
@@ -915,7 +964,7 @@ bool SamplinPlotData::save(const char *fn)
    if((ret=f.open( IO_WriteOnly))==TRUE){
       QDataStream s(&f);
       f.writeBlock(GRAPH_DATA_ID3,strlen(GRAPH_DATA_ID3));
-      filename=tmp;
+      setFileName(tmp);
 
       //checksum
       QBuffer sbuf;
@@ -933,19 +982,24 @@ bool SamplinPlotData::save(const char *fn)
       s << chksum;
       f.close();
       setSaved();
+
    }
    else d_fileerror="Can't open file "+tmp+" for writing.";
+
+   setBusy(FALSE);
    return(ret);
-   
+      
 }
 
-bool SamplinPlotData::exportData(const char *fn)
+bool SamplinPlotData::exportASCII(const char *fn)
 {
    SamplinCurve *crv;
    QString tmp;
    bool ret;
    unsigned int i, max_len,n_ser;
    
+   setBusy(TRUE);
+   
    tmp=fn;
    d_fileerror="";
    QFile f(tmp);
@@ -1080,9 +1134,79 @@ bool SamplinPlotData::exportData(const char *fn)
       f.close();
    }
    else d_fileerror="Can't open file "+tmp+" for writing.";
+
+   setBusy(FALSE);
+   
    return(ret);
 }
 
+bool SamplinPlotData::importASCII(const char *fn)
+{
+   SamplinCurve *crv;
+   QString tmp;
+   bool ret;
+   double values[2];
+   unsigned int i, n_lines=0;
+   int res, columns=0, headers=0,this_columns;
+   static char *sptr;
+   regex_t preg;
+   regmatch_t matches[4];
+   
+   preg.translate=0;
+   preg.allocated=0;
+   preg.buffer=0;
+       
+   setBusy(TRUE);
+   
+   tmp=fn;
+   d_fileerror="";
+   QFile f(tmp);
+   
+   if((ret=f.open( IO_ReadOnly))==TRUE){
+      
+      crv = new SamplinCurve();
+      tmp = "Import ";
+      tmp+= fn;
+      crv->setName(tmp);
+      insertCurve(crv);
+      
+      QTextStream s(&f);
+      res=regcomp(&preg,"([ ;:,\t]*)(([0-9]+|([0-9]*\\.[0-9]*))([eE][-+]?[0-9]+)?)",REG_EXTENDED);
+      if(res)
+	printf("regcomp error :%i\n", index);
+      else {
+	 while(!s.eof()){
+	    tmp=s.readLine();
+//	    printf("Matching line: '%s'\n",tmp.data());
+	    this_columns=0;
+	    sptr=tmp.data();
+	    while(!regexec(&preg,sptr,4,matches,0) && this_columns<2){
+//	       printf("Offsets: %i-%i, string: %.*s\n",matches[0].rm_so,matches[0].rm_eo,matches[0].rm_eo-matches[0].rm_so,sptr);
+
+	       values[this_columns]=atof(sptr);
+	       sptr+=matches[0].rm_eo;
+	       ++this_columns;
+	    }
+	    if(this_columns>columns)columns=this_columns;
+	    if(this_columns==2){
+	       crv->putPoint(-1,values[0],values[1]);
+//	       printf("Adding point: [%lf,%lf]\n",values[0],values[1]);
+	    }
+	    ++n_lines;
+	 }
+//	 printf("Got %i lines with %i columns\n",n_lines,columns);
+	 f.at(0);
+	 
+      }
+      regfree(&preg);
+      f.close();
+      autoRefresh();
+   }
+
+   setBusy(FALSE);
+}
+
+
 void SamplinPlotData::setTitle(const char *t)
 {
    d_title=t;
@@ -1164,17 +1288,17 @@ bool SamplinPlotData::isSaved(void)
    
    if(savedData==sbuf.buffer())return TRUE;
      else {
-/*	ptr1=sbuf.buffer().data();
-	ptr2=savedData.data();
-	size1=sbuf.buffer().size();
-	size2=savedData.size();
-	f1=fopen("data1.cmp","wb");
-	for(i=0;i<size1;++i)fputc(*(ptr1+i),f1);
-	fclose(f1);
-	f2=fopen("data2.cmp","wb");
-	for(i=0;i<size2;++i)fputc(*(ptr2+i),f2);	
-	fclose(f2);
-*/	
+//	ptr1=sbuf.buffer().data();
+//	ptr2=savedData.data();
+//	size1=sbuf.buffer().size();
+//	size2=savedData.size();
+//	f1=fopen("data1.cmp","wb");
+//	for(i=0;i<size1;++i)fputc(*(ptr1+i),f1);
+//	fclose(f1);
+//	f2=fopen("data2.cmp","wb");
+//	for(i=0;i<size2;++i)fputc(*(ptr2+i),f2);	
+//	fclose(f2);
+	
 	return FALSE;
      }
    
