@@ -1,8 +1,8 @@
 /****************************************************************************
 ** GraphGridData meta object code from reading C++ file 'GraphGridData.h'
 **
-** Created: Mon Feb 22 21:44:55 1999
-**      by: The Qt Meta Object Compiler ($Revision: 2.25.2.11 $)
+** Created: Thu Sep 2 20:49:13 1999
+**      by: The Qt Meta Object Compiler ($Revision: 2.25.2.12 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
@@ -34,8 +34,8 @@ void GraphGridData::initMetaObject()
 {
     if ( metaObj )
 	return;
-    if ( strcmp(QDialog::className(), "QDialog") != 0 )
-	badSuperclassWarning("GraphGridData","QDialog");
+    if ( strcmp(QWidget::className(), "QWidget") != 0 )
+	badSuperclassWarning("GraphGridData","QWidget");
 
 #if QT_VERSION >= 200
     staticMetaObject();
@@ -45,13 +45,13 @@ void GraphGridData::staticMetaObject()
 {
     if ( metaObj )
 	return;
-    QDialog::staticMetaObject();
+    QWidget::staticMetaObject();
 #else
 
-    QDialog::initMetaObject();
+    QWidget::initMetaObject();
 #endif
 
-    metaObj = new QMetaObject( "GraphGridData", "QDialog",
+    metaObj = new QMetaObject( "GraphGridData", "QWidget",
 	0, 0,
 	0, 0 );
 }
