@@ -23,14 +23,14 @@
 #ifndef MAIN_H
 #define MAIN_H
 
-#include <qmsgbox.h>
-
-#include <kapp.h>
+#include "devices.h"
 
 #include <string.h>
-//#include <unistd.h>
 #include <stdio.h>
 
-#include <devices.h>
+#include <qmsgbox.h>
+#include <kapp.h>
+
+
 
 #endif // MAIN_H
